#pragma once

#include <Parameter.h>
#include <Factory.h>
#include <slug/spec/Neuron.h>
#include <slug/spec/Synapse.h>
#include <slug/spec/Input.h>
#include <slug/spec/Report.h>

class Recipe
{
public:
	Recipe(const std::string& name);
	const std::string& name() const;
	const std::vector<ParameterBase*>& parameters() const;
private:
	std::vector<ParameterBase*> _parameters;
	std::string _name;
protected:
	void _addParameter(ParameterBase* parameter);
};

template<typename T>
class SpecificRecipe
	: public Recipe
{
public:
	SpecificRecipe(const std::string& name);
	virtual bool generate(unsigned int count) = 0;
	const std::vector<T*>& objects() const;
protected:
	std::vector<T*> _objects;
};

typedef SpecificRecipe<slug::spec::Neuron> NeuronRecipe;
typedef SpecificRecipe<slug::spec::Synapse> SynapseRecipe;
typedef SpecificRecipe<slug::spec::Input> InputRecipe;
typedef SpecificRecipe<slug::spec::Report> ReportRecipe;

typedef FactoryBase<NeuronRecipe> NeuronRecipeFactory;
typedef FactoryBase<SynapseRecipe> SynapseRecipeFactory;
typedef FactoryBase<InputRecipe> InputRecipeFactory;
typedef FactoryBase<ReportRecipe> ReportRecipeFactory;

#include <Recipe.hpp>
