#include <slug/util/BinaryStream.h>

int main(int argc, char** argv)
{
	if (argc != 6)
	{
		printf("Usage: %s <neuronFile> <synapseFile> "
		       "<numNeurons> <numSynapses> <outPrefix>\n", argv[0]);
		return -1;
	}

	//Test if the neuron file is real
	FILE* neuronFile = fopen(argv[1], "r");
	if (NULL == neuronFile)
	{
		printf("Error: Could not open neuron file %s\n", argv[1]);
		return -1;
	}

	//Test if the synapse file is real
	FILE* synapseFile = fopen(argv[2], "r");
	if (NULL == synapseFile)
	{
		printf("Error: Could not open synapse file %s\n", argv[2]);
		fclose(neuronFile);
		return -1;
	}

	unsigned int numNeurons = atoi(argv[3]);
	unsigned int numSynapses = atoi(argv[4]);
	slug::util::OutputStream ns(std::string(argv[5]) + "_neurons.bin");
	ns << numNeurons;
	for (unsigned int i = 0; i < numNeurons; ++i)
	{
		unsigned int id;
		float a;
		float b;
		float c;
		float d;
		float v;
		float u;
		fscanf(neuronFile, "%u %f %f %f %f %f %f",
		       &id, &a, &b, &c, &d, &v, &u);
		ns << id;
		ns << a;
		ns << b;
		ns << c;
		ns << d;
		ns << v;
		ns << u;
	}
	ns.close();

	slug::util::OutputStream ss(std::string(argv[5]) + "_connections.bin");
	ss << numSynapses;
	for (unsigned int i = 0; i < numSynapses; ++i)
	{
		unsigned int preNeuron;
		unsigned int postNeuron;
		unsigned int delay;
		float weight;
		float APlus;
		float AMinus;
		float tauPlus;
		float tauMinus;
		fscanf(synapseFile, "%u %u %u %f %f %f %f %f",
			   &preNeuron, &postNeuron,
			   &delay, &weight, &APlus,
			   &AMinus, &tauPlus, &tauMinus)
		ss << preNeuron;
		ss << postNeuron;
		ss << delay;
		ss << weight;
		ss << APlus;
		ss << AMinus;
		ss << tauPlus;
		ss << tauMinus;
	}

	return 0;
}
