#include <slug/readers/ncs5.h>
#include <slug/spec/Distributor.h>
#include <slug/spec/ClusterWriter.h>

using namespace slug;

int main(int argc, char** argv)
{
	if (argc != 4)
	{
		std::cout << "Usage: " << argv[0] << " <NCS File> <Cluster File> "
			"<Output Directory>" << std::endl;
		return -1;
	}

	std::cout << "Reading input file " << argv[1] << std::endl;
	spec::NeuronList neurons;
	spec::SynapseList synapses;
	spec::InputList inputs;
	spec::ReportList reports;
	auto fileResults =
		readers::ncs5::read(argv[1], neurons, synapses, inputs, reports);
	if (nullptr == fileResults)
	{
		std::cout << "Failed to parse input file" << std::endl;
		return -1;
	}
	std::cout << "Read input file." << std::endl;
	std::cout << neurons.size() << " neurons." << std::endl;
	std::cout << synapses.size() << " synapses." << std::endl;
	std::cout << inputs.size() << " inputs." << std::endl;
	std::cout << reports.size() << " reports." << std::endl;
	std::cout << std::endl;

	std::cout << "Reading cluster file " << argv[2] << std::endl;
	auto cluster = slug::spec::Cluster::build(argv[2]);
	if (nullptr == cluster)
	{
		std::cout << "Failed to read cluster file" << std::endl;
		return -1;
	}
	std::cout << "Read cluster file" << std::endl;
	std::cout << cluster->machines.size() << " machines" << std::endl;
	std::cout << std::endl;
	for (auto machine : cluster->machines)
	{
		std::cout << "\tMachine Name: " << machine->name << std::endl;
		std::cout << "\t\t" << machine->devices.size() << " devices" 
		    << std::endl;
		for (auto device : machine->devices)
		{
			std::cout << "\t\t\tDevice Type: " << device->type() << std::endl;
			std::cout << "\t\t\tPower: " << device->power << std::endl;
			std::cout << std::endl;
		}
		std::cout << std::endl;
	}
	
	std::cout << "Distributing" << std::endl;
	auto distributor = new slug::spec::Distributor();
	if (!distributor->distribute(neurons, synapses, reports, cluster))
	{
		std::cout << "Failed to distribute" << std::endl;
		return -1;
	}
	std::cout << "Distributed elements" << std::endl;

	std::cout << "Finalizing distribution" << std::endl;
	if (!readers::ncs5::finalize(cluster, fileResults))
	{
		std::cout << "Failed to finalize distribution" << std::endl;
		return -1;
	}
	std::cout << "Finalized distribution" << std::endl;

	std::cout << "Writing files" << std::endl;
	auto writer = new slug::spec::ClusterWriter();
	if (!writer->write(cluster, reports, argv[3]))
	{
		std::cout << "Failed to write files" << std::endl;
		return -1;
	}
	std::cout << "Files written" << std::endl;

	return 0;
}
