#include <slug/sim/BrainSimulator.h>
#include <chrono>

using namespace slug;
using namespace slug::sim;

int main(int argc, char** argv)
{
	if (argc < 2)
	{
		std::cerr << "Usage: " << argv[0] << " <inputDirectory>" << std::endl;
		return -1;
	}

	std::string hostname;
	{
		char buf[512];
		gethostname(buf, 512);
		hostname = std::string(buf);
	}

	std::cout << "Loading simulator plugins..." << std::endl;

	auto pluginPath = std::getenv("SLUG_PLUGIN_PATH");
	if (nullptr != pluginPath)
		BrainSimulator::loadPlugins(pluginPath);
	else
		BrainSimulator::loadPlugins("lib/plugins");

	BrainSimulator::initializePlugins();

	std::cout << "Reading input files..." << std::endl;

	BrainSimulator* simulator = new BrainSimulator(hostname);
	if (!simulator->read(argv[1]))
	{
		std::cerr << "Failed to read input directory " << argv[1] << std::endl;
		return -1;
	}

	std::cout << "Initializing simulator..." << std::endl;

	if (!simulator->init(argc, argv))
	{
		std::cerr << "Failed to initialize the simulator." << std::endl;
		return -1;
	}

	std::cout << "Running simulation..." << std::endl;
	simulator->barrier();
	auto start = std::chrono::high_resolution_clock::now();
	if (!simulator->run())
	{
		std::cerr << "Failed to run simulation." << std::endl;
		return -1;
	}
	auto end = std::chrono::high_resolution_clock::now();
	std::chrono::duration<float> dt(end - start);
	std::cout << "Time: " << dt.count() << " seconds" << std::endl;
	std::cout << "Simulation complete." << std::endl;

	delete simulator;

	return 0;
}
