#include <slug/spec/Specification.h>

#include <slug/util/Strings.h>
#include <slug/spec/Distributor.h>
#include <slug/spec/ClusterWriter.h>
#include <slug/spec/Specification.h>
#include <slug/spec/Components.h>
#include <slug/sim/DataBuffer.h>
#include <slug/sim/InputUpdater.h>
#include <slug/sim/VectorExchanger.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>

using namespace slug::spec;
using namespace slug;
using namespace slug::util;
using namespace std;

slug::sim::SpecificDeviceVectorExchanger<MemoryType::CUDA> dve(1,3);

int main(int argc, char** argv)
{
	//Make sure we have enough arguments
	if (argc != 3)
	{
		printf("Usage: %s <neuronFile> <synapseFile>\n", argv[0]);
		return -1;
	}

	//Test if the neuron file is real
	FILE* neuronFile = fopen(argv[1], "r");
	if (NULL == neuronFile)
	{
		printf("Error: Could not open neuron file %s\n", argv[1]);
		return -1;
	}

	//Test if the synapse file is real
	FILE* synapseFile = fopen(argv[2], "r");
	if (NULL == synapseFile)
	{
		printf("Error: Could not open synapse file %s\n", argv[2]);
		fclose(neuronFile);
		return -1;
	}

	//Build-a-brain workshop
	NeuronList neurons;
	SynapseList synapses;
	Unique izhikevichKey = StringID::get("Izhikevich");
	{
		std::map<unsigned int, Neuron*> neuronMap;
		unsigned int id;
		float a;
		float b;
		float c;
		float d;
		float u;
		float v;
		Unique aKey = StringID::get("a");
		Unique bKey = StringID::get("b");
		Unique cKey = StringID::get("c");
		Unique dKey = StringID::get("d");
		Unique uKey = StringID::get("u");
		Unique vKey = StringID::get("v");
		while (7 == fscanf(neuronFile, 
		                   "%u %f %f %f %f %f %f",
						   &id, &a, &b, &c, &d, &v, &u))
		{
			Neuron* n = new Neuron(izhikevichKey);
			Specification* spec = new Specification();
			n->type = izhikevichKey;
			spec->set(aKey, a);
			spec->set(bKey, b);
			spec->set(cKey, c);
			spec->set(dKey, d);
			spec->set(vKey, v);
			spec->set(uKey, u);
			n->specification = spec;
			n->id.user = id;
			neurons.push_back(n);
			neuronMap[id] = n;
		}
		fclose(neuronFile);

		unsigned int preNeuron;
		unsigned int postNeuron;
		unsigned int delay;
		float weight;
		float APlus;
		float AMinus;
		float tauPlus;
		float tauMinus;

		Unique weightKey = StringID::get("weight");
		Unique APlusKey = StringID::get("APlus");
		Unique AMinusKey = StringID::get("AMinus");
		Unique tauPlusKey = StringID::get("tauPlus");
		Unique tauMinusKey = StringID::get("tauMinus");
		Unique STDPKey = StringID::get("STDP");
		while (8 == fscanf(synapseFile,
		                   "%u %u %u %f %f %f %f %f",
						   &preNeuron, &postNeuron,
						   &delay, &weight, &APlus,
						   &AMinus, &tauPlus, &tauMinus))
		{
			Synapse* s = new Synapse(STDPKey);

			//Every synapse must have a presynaptic neuron and postsynaptic 
			//neuron to connect to
			s->connect(neuronMap[preNeuron], neuronMap[postNeuron]);

			s->delay = delay;

			Specification* spec = new Specification();
			spec->set(APlusKey, APlus);
			spec->set(AMinusKey, AMinus);
			spec->set(tauPlusKey, tauPlus);
			spec->set(tauMinusKey, tauMinus);
			spec->set(weightKey, weight);
			s->specification = spec;
			synapses.push_back(s);
		}
		fclose(synapseFile);
	}

	printf("Number of Neurons: %lu\n", neurons.size());
	printf("Number of Synapses: %lu\n", synapses.size());

	//Build a cluster
	Cluster* cluster = new Cluster();
	Machine* machine = new Machine();
	machine->name = "testMachine";
	Device* device0 = new CUDADevice();
	Device* device1 = new CUDADevice();
	machine->devices.push_back(device0);
	machine->devices.push_back(device1);
	cluster->machines.push_back(machine);

	Distributor* distributor = new Distributor();
	ReportList reports;
	distributor->distribute(neurons, synapses, reports, cluster);

	ClusterWriter* writer = new ClusterWriter();
	writer->write(cluster, reports, "foobar");

	return 0;
}

