#include <stdio.h>
#include "CUDA.h"
#include <slug/sim/CUDA.h>

__global__ void linearKernel(const float* baseAmplitude,
                  const float* amplitudeSlope,
			      const float* startTime,
			      const float* endTime,
			      float elapsedTime,
			      const unsigned int* neuronIDs,
                  float* inputCurrent,
			      unsigned int numInputs)
{
	unsigned int index = blockIdx.x * blockDim.x + threadIdx.x;
	unsigned int stride = blockDim.x * gridDim.x;

	for (; index < numInputs; index += stride)
	{
		float amplitude = baseAmplitude[index];
		float _amplitudeSlope = amplitudeSlope[index];
		float _startTime = startTime[index];
		float _endTime = endTime[index];
		if (elapsedTime < _startTime || elapsedTime > _endTime)
			continue;
		amplitude += _amplitudeSlope * (elapsedTime - _startTime);
		atomicAdd(inputCurrent + neuronIDs[index], amplitude);
	}
}

void linear(const float* baseAmplitude,
            const float* amplitudeSlope,
			const float* startTime,
			const float* endTime,
			float elapsedTime,
			const unsigned int* neuronIDs,
            float* inputCurrent,
			unsigned int numInputs)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid =
		(numInputs + threadsPerBlock - 1) / threadsPerBlock;
	linearKernel<<<blocksPerGrid, 
	               threadsPerBlock, 
				   0, 
				   slug::sim::CUDA::stream>>>
		(baseAmplitude,
		 amplitudeSlope,
		 startTime,
		 endTime,
		 elapsedTime,
		 neuronIDs,
		 inputCurrent,
		 numInputs);
}

