#include "CUDA.h"
#include <slug/sim/InputSimulator.h>
#include <slug/sim/CUDA.h>

using namespace slug;
using namespace slug::sim;

template<class Derived, util::MemoryType MType>
class LinearSimulator 
	: public sim::InputSimulator<MType>
{
public:
	virtual bool init(const SimulationProperties* properties)
	{
		return static_cast<Derived*>(this)->init(properties);
	}

	virtual void update(slug::sim::InputBuffer<MType>* buffer)
	{
		static_cast<Derived*>(this)->update(buffer);
	}
};

class CUDALinearSimulator 
	: public LinearSimulator<CUDALinearSimulator, util::MemoryType::CUDA>
{
public:
	static const auto CUDA = util::MemoryType::CUDA;
	static const auto System = util::MemoryType::System;

	bool init(const SimulationProperties* properties)
	{
		_properties = properties;
		_elapsedTime = 0.0f;
		_timestep = float(_properties->timestep) / 1000.0f;

		require("baseAmplitude");
		require("amplitudeSlope");
		require("startTime");
		require("endTime");

		if (!validate())
			return false;

		_baseAmplitude = _allocate<float>(numInputs);
		_transfer<CUDA, System>
			(_baseAmplitude, 
			 getParameterArray<float>("baseAmplitude"), 
			 numInputs);

		_amplitudeSlope = _allocate<float>(numInputs);
		_transfer<CUDA, System>
			(_amplitudeSlope,
			 getParameterArray<float>("amplitudeSlope"),
			 numInputs);

		_startTime = _allocate<float>(numInputs);
		_transfer<CUDA, System>
		    (_startTime,
			 getParameterArray<float>("startTime"),
			 numInputs);

		_endTime = _allocate<float>(numInputs);
		_transfer<CUDA, System>
			(_endTime,
			getParameterArray<float>("endTime"),
			numInputs);

		return true;
	}

	void update(InputBuffer<util::MemoryType::CUDA>* buffer)
	{
		_elapsedTime += _timestep;
		linear(_baseAmplitude,
		       _amplitudeSlope,
			   _startTime,
			   _endTime,
			   _elapsedTime,
			   neuronIDs,
			   buffer->inputCurrent,
			   numInputs);
		slug::sim::CUDA::synchronize();
	}
private:
	const SimulationProperties* _properties;
	float _timestep;
	float _elapsedTime;
	float* _baseAmplitude;
	float* _amplitudeSlope;
	float* _startTime;
	float* _endTime;
};

extern "C"
{

void activate()
{
	static const auto CUDA = util::MemoryType::CUDA;
	auto sim =
		new sim::SpecificFactory<sim::InputSimulator<CUDA>,
		                         CUDALinearSimulator>("linear", "linear", 0);
	CUDAInputSimulators::add(sim);
}

}
