#pragma once
#include <slug/sim/InputSimulator.h>
#include <Mode.h>
#include <curand.h>

template<slug::util::MemoryType MType>
struct MemData
{
	void init();
};

template<>
struct MemData<slug::util::MemoryType::CUDA>
{
	void init();
	curandGenerator_t generator;
};

template<slug::util::MemoryType MType, Mode mode>
class FilebasedSimulator
	: public slug::sim::InputSimulator<MType>
{
public:
	virtual bool init(const slug::sim::SimulationProperties* properties);

	virtual void update(slug::sim::InputBuffer<MType>* buffer);
private:
	struct Bundle
	{
		float* rolls;
		unsigned int* indices;
		unsigned int size;
		float amplitude;
		float* probabilities;
		unsigned int numProbabilities;
		float minRange;
		float maxRange;
		unsigned int sampleRate;
	};
	struct Event
	{
		float startTime;
		float endTime;
		float duration;
		Bundle* bundle;
		unsigned int step;
		unsigned int counter;
	};
	std::list<Event*> _events;
	std::vector<Bundle*> _bundles;
	std::vector<Event*> _storedEvents;
	std::list<Event*> _activeEvents;
	MemData<MType> _memData;
	float _timestep;
	float _elapsedTime;

	void _processEvent(Event* e, slug::sim::InputBuffer<MType>* buffer);
};

#include <Filebased.hpp>
