#include <Filebased.h>
#include <CUDA.h>
#include <stdio.h>
#include <slug/sim/CUDA.h>
#include <atomic>

const auto CUDA = slug::util::MemoryType::CUDA;
const auto System = slug::util::MemoryType::System;

void MemData<CUDA>::init()
{
	curandCreateGenerator(&generator, CURAND_RNG_PSEUDO_DEFAULT);
}

template<>
void MemData<System>::init()
{
}

template<>
void FilebasedSimulator<CUDA, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
	auto bundle = e->bundle;
	if (e->step >= bundle->numProbabilities)
		return;

	//compute firing chance
	auto p = bundle->probabilities[e->step];
	p *= (bundle->maxRange - bundle->minRange);
	p += bundle->minRange;
	p *= _timestep;

	//Roll the dice
	curandSetStream(_memData.generator, slug::sim::CUDA::stream);
	curandGenerateUniform(_memData.generator, bundle->rolls, bundle->size);

	rollCurrent(bundle->indices,
	            bundle->rolls,
				p,
				bundle->amplitude,
				buffer->inputCurrent,
				bundle->size);
	slug::sim::CUDA::synchronize();
	e->counter++;
	if (e->counter >= bundle->sampleRate)
	{
		e->counter = 0;
		e->step++;
	}
}

template<>
void FilebasedSimulator<CUDA, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
}

template<>
void FilebasedSimulator<System, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
}

template<>
void FilebasedSimulator<System, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
}

extern "C"
{

void activate()
{

  {
	  static const auto type = slug::util::MemoryType::CUDA;
	  typedef slug::sim::InputSimulator<type> simType;
	  typedef FilebasedSimulator<type, Mode::Current> currentSim;
	  typedef FilebasedSimulator<type, Mode::Voltage> voltageSim;
	  typedef slug::sim::Factory<simType> factoryType;
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, currentSim>
			  ("ncs5FilebasedCurrent", "ncs5FilebasedCurrent", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, voltageSim>
			  ("ncs5FilebasedVoltage", "ncs5FilebasedVoltage", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
  }

  if (0)
  {
	  static const auto type = slug::util::MemoryType::System;
	  typedef slug::sim::InputSimulator<type> simType;
	  typedef FilebasedSimulator<type, Mode::Current> currentSim;
	  typedef FilebasedSimulator<type, Mode::Voltage> voltageSim;
	  typedef slug::sim::Factory<simType> factoryType;
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, currentSim>
			  ("ncs5FilebasedCurrent", "ncs5FilebasedCurrent", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, voltageSim>
			  ("ncs5FilebasedVoltage", "ncs5FilebasedVoltage", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
  }

}

}

