/*
	Implementation file for Pulse.h
	namespace
*/

template<slug::util::MemoryType MType, Mode mode>
bool PulseSimulator<MType, mode>::
init(const slug::sim::SimulationProperties* properties)
{
	std::cout << "test" << std::endl;
	_timestep = properties->timestep;
	_elapsedTime = 0;

	this->require("amplitude");
	this->require("width");
	this->require("frequency");
	this->require("numSpecs");
	this->require("specIndex");
	this->require("numWindows");
	this->require("windowStarts");
	this->require("windowEnds");

	if (!this->validate())
		return false;

	unsigned int numSpecs = 
		this->template getParameter<unsigned int>("numSpecs");
	std::vector<std::vector<unsigned int>> indexSets;
	indexSets.resize(numSpecs);
	unsigned int* nIDs = new unsigned int[this->numInputs];
	this->template _transfer<slug::util::MemoryType::System, MType>
		(nIDs, this->neuronIDs, this->numInputs);
	unsigned int* specIndices = 
		this->template getParameterArray<unsigned int>("specIndex");
	for (unsigned int i = 0; i < this->numInputs; ++i)
	{
		unsigned int id = nIDs[i];
		unsigned int specIndex = specIndices[i];
		indexSets[specIndex].push_back(id);
	}
	float* amps = this->template getParameterArray<float>("amplitude");
	float* freqs = this->template getParameterArray<float>("frequency");
	float* widths = this->template getParameterArray<float>("width");
	unsigned int* numWindows =
		this->template getParameterArray<unsigned int>("numWindows");
	float* windowStarts =
		this->template getParameterArray<float>("windowStarts");
	float* windowEnds = 
		this->template getParameterArray<float>("windowEnds");
	for (unsigned int i = 0; i < numSpecs; ++i)
	{
		auto bundle = new Bundle;
		bundle->amplitude = amps[i];
		bundle->delay = 1.0f / freqs[i];
		bundle->width = widths[i];
		bundle->size = indexSets[i].size();
		bundle->indices = 
			this->template _allocate<unsigned int>(indexSets[i].size());
		std::sort(indexSets[i].begin(), indexSets[i].end());
		this->template _transfer<MType, slug::util::MemoryType::System>
			(bundle->indices, &(indexSets[i][0]), indexSets[i].size());
		for (unsigned int j = 0; j < numWindows[i]; ++j)
		{
			auto event = new Event;
			event->startTime = windowStarts[j];
			event->endTime = windowEnds[j];
			event->duration = windowEnds[j] - windowStarts[j];
			event->bundle = bundle;
			_storedEvents.push_back(event);
		}
		windowStarts += numWindows[i];
		windowEnds += numWindows[i];
		_bundles.push_back(bundle);
	}
	auto Earlier = [](const Event* a, const Event* b)
	{
		return a->startTime < b->startTime;
	};
	std::sort(_storedEvents.begin(), _storedEvents.end(), Earlier);
	for (auto e : _storedEvents)
		_events.push_back(e);
	std::cout << _events.size() << " events" << std::endl;

	delete [] nIDs;
	return true;
}

template<slug::util::MemoryType MType, Mode mode>
void PulseSimulator<MType, mode>::update(slug::sim::InputBuffer<MType>* buffer)
{
	//Increment time
	_elapsedTime += _timestep;
	//Remove finished events
	auto Finished = [_elapsedTime](const Event* e)
	{
		return _elapsedTime >= e->endTime;
	};
	_activeEvents.remove_if(Finished);

	//Add newly started events
	for (auto& e : _events)
	{
		if (_elapsedTime >= e->startTime)
		{
			e->elapsed = 0.0f;
			_activeEvents.push_back(e);
		}
		else
			break;
	}

	//Remove the added events from the original list
	auto Started = [_elapsedTime](const Event* e)
	{
		return _elapsedTime >= e->startTime;
	};
	_events.remove_if(Started);

	for (auto& e : _activeEvents)
	{
		_processEvent(e, buffer);
	}
}

