#include <stdio.h>
#include <CUDA.h>
#include <slug/sim/CUDA.h>
#include <slug/util/CUDA.h>
#include <slug/util/Debug.h>

__global__ void currentKernel(unsigned int* indices, 
                              float value,
							  float* currents,
							  unsigned int numIndices)
{
	unsigned int index = grid::thread();
	unsigned int stride = grid::stride();
	for (; index < numIndices; index += stride)
	{
		unsigned int i = indices[index];
		atomicAdd(currents + i, value);
	}
}

__global__ void voltageKernel(unsigned int* indices,
                              float value,
							  float* voltages,
							  unsigned int* clampVector,
							  unsigned int numIndices)
{
	unsigned int index = grid::thread();
	unsigned int stride = grid::stride();
	for (; index < numIndices; index += stride)
	{
		unsigned int i = indices[index];
		voltages[i] = value;
		unsigned int word = bit::word(i);
		unsigned int mask = bit::mask(i);
		atomicOr(clampVector + word, mask);
	}
}

void addCurrent(unsigned int* indices,
                float value,
				float* currents,
				unsigned int numIndices)
{
	unsigned int threadsPerBlock = 128;
	unsigned int blocksPerGrid = 
		(numIndices + threadsPerBlock - 1) / threadsPerBlock;
	unsigned int shmemSize = 0;
	cudaErrorCheck();
	currentKernel<<<threadsPerBlock,
	                blocksPerGrid,
					shmemSize,
					slug::sim::CUDA::stream>>>
		(indices,
		 value,
		 currents,
		 numIndices);
	cudaErrorCheck();
}

void setVoltage(unsigned int* indices,
                float value,
				float* voltages,
				unsigned int* clampVector,
				unsigned int numIndices)
{
	unsigned int threadsPerBlock = 128;
	unsigned int blocksPerGrid = 
		(numIndices + threadsPerBlock - 1) / threadsPerBlock;
	unsigned int shmemSize = 0;
	cudaErrorCheck();
	voltageKernel<<<threadsPerBlock,
	                blocksPerGrid,
					shmemSize,
					slug::sim::CUDA::stream>>>
		(indices,
		 value,
		 voltages,
		 clampVector,
		 numIndices);
	cudaErrorCheck();
}
