#include <CUDA.h>
#include <Linear.h>
#include <stdio.h>
#include <slug/sim/CUDA.h>
#include <atomic>

const auto CUDA = slug::util::MemoryType::CUDA;
template<>
void LinearSimulator<CUDA, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
	float r = (_elapsedTime - e->startTime) / e->duration;
	float v = e->bundle->startingAmplitude + r * e->bundle->slope;
	addCurrent(e->bundle->indices, v, buffer->inputCurrent, e->bundle->size);
	slug::sim::CUDA::synchronize();
}

template<>
void LinearSimulator<CUDA, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
	float r = (_elapsedTime - e->startTime) / e->duration;
	float v = e->bundle->startingAmplitude + r * e->bundle->slope;
	setVoltage(e->bundle->indices, 
	           v, 
			   buffer->clampVoltage, 
			   buffer->voltageClamp, 
			   e->bundle->size);
	slug::sim::CUDA::synchronize();
}

const auto System = slug::util::MemoryType::System;
template<>
void LinearSimulator<System, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
	float r = (_elapsedTime - e->startTime) / e->duration;
	float v = e->bundle->startingAmplitude + r * e->bundle->slope;
	auto size = e->bundle->size;
	auto current = buffer->inputCurrent;
	auto indices = e->bundle->indices;
	std::unique_lock<std::mutex> lock(buffer->writeLock);
	for (unsigned int i = 0; i < size; ++i)
	{
		current[indices[i]] += v;
	}
}

template<>
void LinearSimulator<System, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
	float r = (_elapsedTime - e->startTime) / e->duration;
	float v = e->bundle->startingAmplitude + r * e->bundle->slope;
	auto size = e->bundle->size;
	auto clampVoltage = buffer->clampVoltage;
	auto voltageClamp = buffer->voltageClamp;
	auto indices = e->bundle->indices;
	std::unique_lock<std::mutex> lock(buffer->writeLock);
	for (unsigned int i = 0; i < size; ++i)
	{
		unsigned int index = indices[i];
		clampVoltage[index] = v;
		unsigned int word = index >> 5;
		unsigned int mask = 0x80000000 >> (index & 0x1F);
		voltageClamp[word] |= mask;
	}
}
