#include <slug/sim/CUDA.h>
#include <CUDA.h>
#include <Pulse.h>
#include <stdio.h>

const auto CUDA = slug::util::MemoryType::CUDA;
template<>
void PulseSimulator<CUDA, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
	e->elapsed += _timestep;
	if (e->elapsed >= e->bundle->delay)
	{
		e->elapsed -= e->bundle->delay;
	}
	if (e->elapsed <= e->bundle->width)
	{
		addCurrent(e->bundle->indices,
		           e->bundle->amplitude,
				   buffer->inputCurrent,
				   e->bundle->size);
		slug::sim::CUDA::synchronize();
	}
}

template<>
void PulseSimulator<CUDA, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
	e->elapsed += _timestep;
	if (e->elapsed >= e->bundle->delay)
	{
		e->elapsed -= e->bundle->delay;
	}
	if (e->elapsed <= e->bundle->width)
	{
		setVoltage(e->bundle->indices,
		           e->bundle->amplitude,
				   buffer->clampVoltage,
				   buffer->voltageClamp,
				   e->bundle->size);
		slug::sim::CUDA::synchronize();
	}
}

const auto System = slug::util::MemoryType::System;
template<>
void PulseSimulator<System, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
	e->elapsed += _timestep;
	if (e->elapsed >= e->bundle->delay)
	{
		e->elapsed -= e->bundle->delay;
	}
	if (e->elapsed <= e->bundle->width)
	{
		auto size = e->bundle->size;
		auto current = buffer->inputCurrent;
		auto indices = e->bundle->indices;
		auto amp = e->bundle->amplitude;
		std::unique_lock<std::mutex> lock(buffer->writeLock);
		for (unsigned int i = 0; i < size; ++i)
		{
			current[indices[i]] += amp;
		}
	}
}

template<>
void PulseSimulator<System, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
	e->elapsed += _timestep;
	if (e->elapsed >= e->bundle->delay)
	{
		e->elapsed -= e->bundle->delay;
	}
	if (e->elapsed <= e->bundle->width)
	{
		auto size = e->bundle->size;
		auto clampVoltage = buffer->clampVoltage;
		auto voltageClamp = buffer->voltageClamp;
		auto indices = e->bundle->indices;
		auto amp = e->bundle->amplitude;
		std::unique_lock<std::mutex> lock(buffer->writeLock);
		for (unsigned int i = 0; i < size; ++i)
		{
			unsigned int index = indices[i];
			clampVoltage[index] = amp;
			unsigned int word = index >> 5;
			unsigned int mask = 0x80000000 >> (index & 0x1F);
			voltageClamp[word] |= mask;
		}
	}
}

