#include <slug/util/CUDA.h>
#include <slug/sim/CUDA.h>
#include <slug/util/Debug.h>
#include <CUDA.h>
#include <stdio.h>

__global__ 
void currentKernel(unsigned int* indices,
				   float* rolls,
                   float probability,
				   float amplitude,
				   float* currents,
				   unsigned int numIndices)
{
	unsigned int index = grid::thread();
	unsigned int stride = grid::stride();
	for (; index < numIndices; index += stride)
	{
		if (rolls[index] < probability)
		{
			unsigned int i = indices[index];
			atomicAdd(currents + i, amplitude);
		}
	}
}

__global__
void voltageKernel(unsigned int* indices,
				   float* rolls,
				   float probability,
				   float amplitude,
				   float* voltages,
				   unsigned int* clampVector,
				   unsigned int numIndices)
{
	unsigned int index = grid::thread();
	unsigned int stride = grid::stride();
	for (; index < numIndices; index += stride)
	{
		if (rolls[index] < probability)
		{
			unsigned int i = indices[index];
			voltages[i] = amplitude;
			unsigned int word = bit::word(i);
			unsigned int mask = bit::mask(i);
			atomicOr(clampVector + word, mask);
		}
	}
}

void rollCurrent(unsigned int* indices,
				 float* rolls,
                 float probability,
				 float amplitude,
				 float* currents,
				 unsigned int numIndices)
{
	unsigned int threadsPerBlock = 128;
	unsigned int blocksPerGrid = 
		(numIndices + threadsPerBlock - 1) / threadsPerBlock;
	unsigned int shmemSize = 0;
	cudaErrorCheck();
	currentKernel<<<threadsPerBlock,
	                blocksPerGrid,
					shmemSize,
					slug::sim::CUDA::stream>>>
		(indices,
		 rolls,
		 probability,
		 amplitude,
		 currents,
		 numIndices);
	cudaErrorCheck();
}

void rollVoltage(unsigned int* indices,
				 float* rolls,
                 float probability,
				 float amplitude,
				 float* voltages,
				 unsigned int* clampVector,
				 unsigned int numIndices)
{	
	unsigned int threadsPerBlock = 128;
	unsigned int blocksPerGrid = 
		(numIndices + threadsPerBlock - 1) / threadsPerBlock;
	unsigned int shmemSize = 0;
	cudaErrorCheck();
	voltageKernel<<<threadsPerBlock,
	                blocksPerGrid,
					shmemSize,
					slug::sim::CUDA::stream>>>
		(indices,
		 rolls,
		 probability,
		 amplitude,
		 voltages,
		 clampVector,
		 numIndices);
	cudaErrorCheck();
}
