#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <Connection.h>
#include <string.h>
#include <sstream>

Connections::ServerToPortMap Connections::_servers;
std::mutex Connections::_lock;

Connection* Connections::get(const std::string& server,
                             unsigned int port,
							 unsigned int numColumns,
							 float timeInverval,
							 const std::string& name)
{
	std::unique_lock<std::mutex> lock(_lock);
	auto connection = _servers[server][port][name];
	if (connection == nullptr)
	{
		connection = new Connection();
		if (!connection->connect(server, port, numColumns, timeInverval, name))
		{
			delete connection;
			return nullptr;
		}
		_servers[server][port][name] = connection;
	}
	return connection;
}

bool Connection::connect(const std::string& server,
                         unsigned int port,
						 unsigned int numColumns,
						 float timeInterval,
						 const std::string& name)
{
	_numColumns = numColumns;
	_timeInterval = timeInterval;
	struct sockaddr serverAddress;
	struct sockaddr_in* dummyAddress;
	struct hostent* host;
	int socketDescriptor;

	/* Can specify host as either dotted address or name */

	memset((char *) &serverAddress, 0, sizeof (struct sockaddr_in));
	dummyAddress = (struct sockaddr_in *)&serverAddress;
	dummyAddress->sin_family = AF_INET;

	host = gethostbyname(server.c_str());
	if (host == nullptr)
	{
		std::cerr << "Failed to find host " << server << std::endl;
		return false;
	}
	dummyAddress->sin_family = host->h_addrtype;
	memcpy(host->h_addr, 
	       (caddr_t)&dummyAddress->sin_addr.s_addr, 
		   host->h_length);

	dummyAddress->sin_port = htons(port);

	if ((socketDescriptor = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		std::cerr << "Failed to create a socket: " << strerror(errno) <<
			std::endl;
		return false;
	}

	if (::connect(socketDescriptor, &serverAddress, sizeof(struct sockaddr_in)) < 0)
	{
		std::cerr << "Failed to open connection with error " << 
			strerror(errno) << std::endl;
		return false;
	}
	_socketDescriptor = socketDescriptor;

	write(_socketDescriptor, "request\n", 8);
	std::ostringstream ss;
	ss << "read\nbinary\n";
	ss << name << "\n";
	std::string request = ss.str();
	write(_socketDescriptor, request.c_str(), request.length());
	std::cout << request << std::endl;
	return true;
}

void Connection::run(unsigned int numSteps)
{
	auto func = [this, numSteps]()
	{
		for (unsigned int i = 0; i < 3; ++i)
		{
			auto blank = new ProbabilityBuffer(this->_numColumns);
			_addBlank(blank);
		}
		unsigned int numSamples = numSteps / (this->_timeInterval * 1000);
		for (unsigned int i = 0; i < numSamples; ++i)
		{
			auto blank = this->_getBlank();
			read(_socketDescriptor, 
			     blank->probabilities, 
				 sizeof(float) * _numColumns);
			this->publish(blank);
		}
	};

	auto threadFunc = [this, func]()
	{
		_myThread = std::thread(func);
	};
	std::call_once(_once, threadFunc);
}
