#include <Socketbased.h>
#include <CUDA.h>
#include <stdio.h>
#include <slug/sim/CUDA.h>
#include <atomic>

const auto CUDA = slug::util::MemoryType::CUDA;
const auto System = slug::util::MemoryType::System;

void MemData<CUDA>::init()
{
	curandCreateGenerator(&generator, CURAND_RNG_PSEUDO_DEFAULT);
}

template<>
void MemData<System>::init()
{
}

template<>
void SocketbasedSimulator<CUDA, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
	auto bundle = e->bundle;

	//compute firing chance
	//Roll the dice
	float p = bundle->currentValue;
	p *= (bundle->maxRange - bundle->minRange);
	p += bundle->minRange;
	p *= _timestep;

	curandSetStream(_memData.generator, slug::sim::CUDA::stream);
	curandGenerateUniform(_memData.generator, bundle->rolls, bundle->size);

	rollCurrent(bundle->indices,
	            bundle->rolls,
				p,
				bundle->amplitude,
				buffer->inputCurrent,
				bundle->size);

	slug::sim::CUDA::synchronize();

	e->elapsedTime += _timestep;
	if (e->elapsedTime >= bundle->sampleRate)
	{
		auto buf = bundle->sub->pull();
		bundle->currentValue = buf->probabilities[bundle->column];
		e->elapsedTime -= bundle->sampleRate;
		buf->release();
	}
}

template<>
void SocketbasedSimulator<CUDA, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<CUDA>* buffer)
{
}

template<>
void SocketbasedSimulator<System, Mode::Current>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
}

template<>
void SocketbasedSimulator<System, Mode::Voltage>::
_processEvent(Event* e, slug::sim::InputBuffer<System>* buffer)
{
}

extern "C"
{

void activate()
{

  {
	  static const auto type = slug::util::MemoryType::CUDA;
	  typedef slug::sim::InputSimulator<type> simType;
	  typedef SocketbasedSimulator<type, Mode::Current> currentSim;
	  typedef SocketbasedSimulator<type, Mode::Voltage> voltageSim;
	  typedef slug::sim::Factory<simType> factoryType;
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, currentSim>
			  ("ncs5SocketbasedCurrent", "ncs5SocketbasedCurrent", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, voltageSim>
			  ("ncs5SocketbasedVoltage", "ncs5SocketbasedVoltage", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
  }

  if (0)
  {
	  static const auto type = slug::util::MemoryType::System;
	  typedef slug::sim::InputSimulator<type> simType;
	  typedef SocketbasedSimulator<type, Mode::Current> currentSim;
	  typedef SocketbasedSimulator<type, Mode::Voltage> voltageSim;
	  typedef slug::sim::Factory<simType> factoryType;
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, currentSim>
			  ("ncs5SocketbasedCurrent", "ncs5SocketbasedCurrent", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
	  {
		  auto sim =
			  new slug::sim::SpecificFactory<simType, voltageSim>
			  ("ncs5SocketbasedVoltage", "ncs5SocketbasedVoltage", 0);
		  slug::sim::Manager<factoryType>::add(sim);
	  }
  }

}

}

