#include "CUDA.h"
#include <slug/sim/CUDA.h>

__global__ void echoKernel(const float* inputCurrent,
                           unsigned int* fireVector,
						   float* neuronVoltage,
						   unsigned int numNeurons)
{
	unsigned int index = blockDim.x * blockIdx.x + threadIdx.x;
	unsigned int stride = blockDim.x * gridDim.x;
	for (; index < numNeurons; index += stride)
	{
		neuronVoltage[index] = inputCurrent[index];
		if (threadIdx.x & 0x1)
			fireVector[index >> 5] = 0x00000000;
	}
}

void echo(const float* inputCurrent,
          unsigned int* fireVector,
		  float* neuronVoltage,
		  unsigned int numNeurons)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid =
		(numNeurons + threadsPerBlock - 1) / threadsPerBlock;
	echoKernel<<<blocksPerGrid, threadsPerBlock, 0, slug::sim::CUDA::stream>>>
		(inputCurrent, fireVector, neuronVoltage, numNeurons);
}
