#include "CUDA.h"
#include <stdio.h>
#include <slug/sim/CUDA.h>
#include <slug/util/CUDA.h>

void __global__ izhikevichKernel(float* a,
                                 float* b,
				                 float* c,
				                 float* d,
				                 const float* oldU,
								 float* newU,
				                 const float* oldV,
				                 float* newV,
				                 unsigned int* neuronFireVector,
				                 float* synapticCurrent,
								 const float* inputCurrent,
				                 float* threshold,
				                 float dt,
				                 unsigned int numNeurons)
{
	unsigned int index = grid::thread();
	unsigned int stride = grid::stride();
	extern __shared__ unsigned int fireVector[];
	unsigned int& warpResult = fireVector[threadIdx.x];
	unsigned int* resultVectorBase = fireVector + warp::index() * 32;
	unsigned int warpThread = warp::thread();
	unsigned int limit = ((numNeurons + 31) / 32) * 32;
	unsigned int mask = bit::mask(warpThread);
	while (index < limit)
	{
		warpResult = 0;
		if (index < numNeurons)
		{
			//Load parameters
			float _a = a[index];
			float _b = b[index];
			float _c = c[index];
			float _d = d[index];
			float _u = oldU[index];
			float _v = oldV[index];

			//Gather incoming synaptic current
			float I = synapticCurrent[index];
			I += inputCurrent[index];

			//Update parameters
			float _newV = _v;
			float _newU = _u;

			if (_newV >= threshold[index])
			{
				_newV = _c;
				_newU += _d;
				warpResult = mask;
				_v = _newV;
				_u = _newU;
			}

			float _v2 = _v * _v;
			_newV += 0.5f * (dt * (0.04f * _v2 + 5.0f * _v + 140.0f - _u + I));
			_newU += 0.5f * (dt * (_a * (_b * _v - _u)));

			_v = _newV;
			_u = _newU;

			_v2 = _v * _v;
			_newV += 0.5f * (dt * (0.04f * _v2 + 5.0f * _v + 140.0f - _u + I));
			_newU += 0.5f * (dt * (_a * (_b * _v - _u)));

			if (_newV >= threshold[index])
				_newV = threshold[index];

			newV[index] = _newV;
			newU[index] = _newU;
			//printf("%f %f %f %f\n", I, _newV, _newU, dt);
		}
		warp::reduceOr(resultVectorBase, warpThread);
		if (warp::leader())
		{
			neuronFireVector[bit::word(index)] = warpResult;
		}
		index += stride;
	}
}

void izhikevich(float* a,
                float* b,
				float* c,
				float* d,
				const float* oldU,
				float* newU,
				const float* oldV,
				float* newV,
				unsigned int* neuronFireVector,
				float* synapticCurrent,
				const float* inputCurrent,
				float* threshold,
				float dt,
				unsigned int numNeurons)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid =
		(numNeurons + threadsPerBlock - 1) / threadsPerBlock;
	size_t shmemSize = threadsPerBlock * sizeof(unsigned int);
	izhikevichKernel<<<blocksPerGrid, 
	                   threadsPerBlock, 
				       shmemSize, 
				       slug::sim::CUDA::stream>>>
		(a,
		 b,
		 c,
		 d,
		 oldU,
		 newU,
		 oldV,
		 newV,
		 neuronFireVector,
		 synapticCurrent,
		 inputCurrent,
		 threshold,
		 dt * 1000.0f,
		 numNeurons);
}
