/*
	Implementation file for ChannelSimulator.h
	namespace 
*/

template<slug::util::MemoryType MType>
bool ChannelSimulator<MType>::read(const void* p)
{
	Parameterized::read(p);

	this->require("numChannels");
	this->require("compartmentID");
	this->require("mPower");
	this->require("unitaryG");
	this->require("strength");
	this->require("mInitial");
	this->require("reversalPotential");

	if (!this->validate())
		return false;

	numChannels = this->template getParameter<unsigned int>("numChannels");
	compartmentIDs = 
		this->template loadParameter<unsigned int>("compartmentID");
	mPower = this->template loadParameter<float>("mPower");
	unitaryG = this->template loadParameter<float>("unitaryG");
	strength = this->template loadParameter<float>("strength");
	m = this->template loadParameter<float>("mInitial");
	reversalPotential = 
		this->template loadParameter<float>("reversalPotential");

	return true;
}

template<slug::util::MemoryType MType>
template<typename T>
T* ChannelSimulator<MType>::loadParameter(const std::string& name)
{
	T* result = this->template _allocate<T>(this->numChannels);
	this->template _transfer<MType, slug::util::MemoryType::System>
		(result, this->template getParameterArray<T>(name), this->numChannels);
	return result;
}

template<slug::util::MemoryType MType>
ChannelSimulator<MType>::~ChannelSimulator()
{
	this->_deallocate(compartmentIDs);
	this->_deallocate(mPower);
	this->_deallocate(unitaryG);
	this->_deallocate(strength);
	this->_deallocate(m);
	this->_deallocate(reversalPotential);
}
