#pragma once
#include <slug/sim/NeuronSimulator.h>
#include <KmChannel.h>
#include <KahpChannel.h>
#include <KaChannel.h>
#include <ChannelUpdater.h>
#include <CompartmentStateBuffer.h>
#include <ChannelCurrentBuffer.h>
#include <slug/sim/CUDA.h>

template<slug::util::MemoryType MType>
class CompartmentSimulator
	: public slug::sim::NeuronSimulator<MType>,
	  public slug::sim::SpecificPublisher<CompartmentStateBuffer<MType>>
{
public:
	CompartmentSimulator();

	virtual bool init(const slug::sim::SimulationProperties* properties);

	virtual void 
	initVoltages(float* voltages,
	             const slug::sim::SimulationProperties* properties);

	typedef typename slug::sim::NeuronSimulator<MType>::UpdateParameters
		UpdateParameters;
	virtual void update(UpdateParameters& parameters);

private:
	template<typename T> T* loadParameter(const std::string& name);

	///Time left after spiking where compartment follows the spike shape
	int* _refractoryTime;
	///Parameter: modifies the amount of current leaking from the compartment
	float* _leakConductance;
	///Parameter: voltage where deviation causes leakage
	float* _leakReversal;
	///Parameter: resting voltage of the compartment
	float* _restVoltage;
	///Computed: amount of voltage above resting that persists
	float* _persistence;
	///Computed: time / capacitance of the compartment
	float* _dtC;
	///Parameter: voltage threshold for firing
	float* _threshold;
	///Parameter: the length of the spike shape for each compartment
	int* _spikeShapeLength;
	///Computed: Pointers into the spike shape data for each compartment
	const float** _spikeShapes;
	///Parameter: All spike shape values
	float* _spikeShapeData;
	///Parameter: amount to increment calcium when spikes occur
	float* _caSpikeIncrement;
	///Computed: amount of calcium that persists over updates
	float* _caPersistence;

	///Updater object for channels - subscribe and publish to
	ChannelUpdater<MType>* _channelUpdater;

	///Future to hold the main channel updater thread
	std::future<bool> _channelUpdaterResult;

	///Subscription to the contributions of channels
	slug::sim::Subscription<ChannelCurrentBuffer<MType>>* _channelSubscription;

	///Self-subscription to our own internal state
	slug::sim::Subscription<CompartmentStateBuffer<MType>>* 
		_compartmentSubscription;

	///The simulation timestep in seconds
	float _timestep;
};

#include <CompartmentSimulator.hpp>
