/*
	Implementation file for KaChannel.h
	namespace
*/

template<slug::util::MemoryType MType>
bool KaChannelSimulator<MType>::
init(const slug::sim::SimulationProperties* properties)
{
	this->require("tauMIndices");
	this->require("tauMVoltage");
	this->require("tauMValue");
	this->require("eHalfMinM");
	this->require("slopeFactorM");

	this->require("tauHIndices");
	this->require("tauHVoltage");
	this->require("tauHValue");
	this->require("eHalfMinH");
	this->require("slopeFactorH");

	this->require("hInitial");
	this->require("hPower");

	if (!this->validate())
		return false;
	
	const slug::util::MemoryType System = slug::util::MemoryType::System;
		
	unsigned int* tauMIndices = 
		this->template getParameterArray<unsigned int>("tauMIndices");
	_tauMIndices = 
		this->template _allocate<unsigned int>(this->numChannels + 1);
	this->template _transfer<MType, System>
		(_tauMIndices, tauMIndices, this->numChannels + 1);
	unsigned int numMValues = tauMIndices[this->numChannels];

	_tauMVoltage = this->template _allocate<float>(numMValues);
	this->template _transfer<MType, System>
		(_tauMVoltage, 
		 this->template getParameterArray<float>("tauMVoltage"),
		 numMValues);

	_tauMValue = this->template _allocate<float>(numMValues);
	this->template _transfer<MType, System>
		(_tauMValue, 
		 this->template getParameterArray<float>("tauMValue"),
		 numMValues);

	_eHalfMinM = this->template loadParameter<float>("eHalfMinM");
	_slopeFactorM = this->template loadParameter<float>("slopeFactorM");
	
	unsigned int* tauHIndices = 
		this->template getParameterArray<unsigned int>("tauHIndices");
	_tauHIndices = 
		this->template _allocate<unsigned int>(this->numChannels + 1);
	this->template _transfer<MType, System>
		(_tauHIndices, tauHIndices, this->numChannels + 1);
	unsigned int numHValues = tauHIndices[this->numChannels];

	_tauHVoltage = this->template _allocate<float>(numHValues);
	this->template _transfer<MType, System>
		(_tauHVoltage, 
		 this->template getParameterArray<float>("tauHVoltage"),
		 numHValues);

	_tauHValue = this->template _allocate<float>(numHValues);
	this->template _transfer<MType, System>
		(_tauHValue, 
		 this->template getParameterArray<float>("tauHValue"),
		 numHValues);

	_eHalfMinH = this->template loadParameter<float>("eHalfMinH");
	_slopeFactorH = this->template loadParameter<float>("slopeFactorH");

	_h = this->template loadParameter<float>("hInitial");
	_hPower = this->template loadParameter<float>("hPower");

	_timestep = properties->timestep;

	return true;
}
