#include <KaChannel.h>
#include <KaChannel.cuh>
#include <slug/sim/CUDA.h>
#include <cmath>

template<>
void KaChannelSimulator<slug::util::MemoryType::CUDA>::
	update(UpdateParameters& parameters)
{
	CUDAKaChannel(compartmentIDs,
	              parameters.compartmentVoltages,
				  _tauMIndices,
				  _tauMVoltage,
				  _tauMValue,
				  _eHalfMinM,
				  _slopeFactorM,
				  _tauHIndices,
				  _tauHVoltage,
				  _tauHValue,
				  _eHalfMinH,
				  _slopeFactorH,
				  unitaryG,
				  strength,
				  mPower,
				  _hPower,
				  reversalPotential,
				  m,
				  _h,
				  parameters.compartmentCurrents,
				  _timestep,
				  numChannels);
	slug::sim::CUDA::synchronize();
}

template<>
void KaChannelSimulator<slug::util::MemoryType::System>::
	update(UpdateParameters& parameters)
{
	for (unsigned int index = 0; index < numChannels; ++index)
	{
		unsigned int compartmentID = compartmentIDs[index];
		float compartmentVoltage = parameters.compartmentVoltages[compartmentID];
		unsigned int tauMIndex = _tauMIndices[index];
		unsigned int tauMEnd = _tauMIndices[index + 1];
		float t_m = 0.0f;
		for (; tauMIndex < tauMEnd; ++tauMIndex)
		{
			if (compartmentVoltage < _tauMVoltage[tauMIndex])
			{
				t_m = _tauMValue[tauMIndex];
				break;
			}
		}
		float m_oo = 
			1.0f / (1.0f + exp(-(compartmentVoltage - _eHalfMinM[index]) / 
			                   _slopeFactorM[index]));
		float _m = m[index];
		_m += (m_oo - _m) * _timestep / t_m;
		_m = (_m < 0.0f)? 0.0f : (_m > 1.0f)? 1.0f : _m;
		m[index] = _m;

		unsigned int tauHIndex = _tauHIndices[index];
		unsigned int tauHEnd = _tauHIndices[index + 1];
		float t_h = 0.0f;
		for (; tauHIndex < tauHEnd; ++tauHIndex)
		{
			if (compartmentVoltage < _tauHVoltage[tauHIndex])
			{
				t_h = _tauHValue[tauHIndex];
				return;
			}
		}
		float h_oo =
			1.0f / (1.0f + exp((compartmentVoltage - _eHalfMinH[index]) /
			                   _slopeFactorH[index]));
		float h = _h[index];
		h += (h_oo - h) * _timestep / t_h;
		h = (h < 0.0f)? 0.0f : (h > 1.0f)? 1.0f : h;
		_h[index] = h;

		float I = unitaryG[index] * strength[index] * pow(_m, mPower[index]) *
		          pow(h, _hPower[index]) * 
				  (reversalPotential[index] - compartmentVoltage);
		parameters.compartmentCurrents[compartmentID] += I;
	}
}

