#include <KahpChannel.h>
#include <KahpChannel.cuh>
#include <slug/sim/CUDA.h>
#include <cmath>

template<>
void KahpChannelSimulator<slug::util::MemoryType::CUDA>::
	update(UpdateParameters& parameters)
{
	CUDAKahpChannel(compartmentIDs,
	                parameters.compartmentVoltages,
					parameters.compartmentCalcium,
					_caScaleFactor,
					_caExpFactor,
					_caHalfMin,
					_caTauScaleFactor,
					unitaryG,
					strength,
					mPower,
					reversalPotential,
					m,
					parameters.compartmentCurrents,
					_timestep,
					numChannels);
	slug::sim::CUDA::synchronize();

}

template<>
void KahpChannelSimulator<slug::util::MemoryType::System>::
	update(UpdateParameters& parameters)
{
	for (unsigned int index = 0; index < numChannels; ++index)
	{
		unsigned int compartmentID = compartmentIDs[index];
		float compartmentVoltage = parameters.compartmentVoltages[compartmentID];
		float funct_m = 
			_caScaleFactor[index] * 
			pow(parameters.compartmentCalcium[compartmentID], _caExpFactor[index]);
		float denominator = (funct_m + _caHalfMin[index]);
		float t_m = _caTauScaleFactor[index] / denominator;
		float m_oo = funct_m / denominator;
		float _m = m[index];
		_m += (m_oo - _m) * _timestep / t_m;
		_m = (_m < 0.0f)? 0.0f : (_m > 1.0f)? 1.0f : _m;
		m[index] = _m;
		float I = unitaryG[index] * strength[index] * pow(_m, mPower[index]) *
		          (reversalPotential[index] - compartmentVoltage);
		parameters.compartmentCurrents[compartmentID] += I;
	}
}
