#include <KmChannel.h>
#include <KmChannel.cuh>
#include <slug/sim/CUDA.h>
#include <cmath>

template<>
void KmChannelSimulator<slug::util::MemoryType::CUDA>::
	update(UpdateParameters& parameters)
{
	CUDAKmChannel(compartmentIDs,
	              parameters.compartmentVoltages,
				  _eHalfMinM,
				  _tauScaleFactorM,
				  _slopeFactorM[0],
				  _slopeFactorM[1],
				  _slopeFactorM[2],
				  unitaryG,
				  strength,
				  mPower,
				  reversalPotential,
				  m,
				  parameters.compartmentCurrents,
				  _timestep,
				  numChannels);
	slug::sim::CUDA::synchronize();
}

template<>
void KmChannelSimulator<slug::util::MemoryType::System>::
	update(UpdateParameters& parameters)
{
	for (unsigned int index = 0; index < numChannels; ++index)
	{
		unsigned int compartmentID = compartmentIDs[index];
		float compartmentVoltage = parameters.compartmentVoltages[compartmentID];
		float dV = compartmentVoltage - _eHalfMinM[index];
		float t_m = _tauScaleFactorM[index] / (exp(dV / _slopeFactorM[0][index]) +
		                                      exp(-dV / _slopeFactorM[1][index]));
		float m_oo = 1.0f / (1.0f + exp(-dV / _slopeFactorM[2][index]));
		float _m = m[index];
		_m += (m_oo - _m) * _timestep / t_m;
		_m = (_m < 0.0f)? 0.0f : (_m > 1.0f)? 1.0f : _m;
		m[index] = _m;
		float I = unitaryG[index] * strength[index] * pow(_m, mPower[index]) *
		          (reversalPotential[index] - compartmentVoltage);
		parameters.compartmentCurrents[compartmentID] += I;
	}
}
