#include <stdio.h>
#include "CUDA.h"
#include <slug/sim/CUDA.h>
#include <slug/util/CUDA.h>

void __global__ pulseKernel(float* delay,
                            const float* interval,
				            unsigned int* neuronFireVector,
				            float* neuronVoltageVector,
				            float dt,
				            unsigned int numNeurons)
{
	unsigned int index = blockIdx.x * blockDim.x + threadIdx.x;
	unsigned int stride = blockDim.x * gridDim.x;
	extern __shared__ unsigned int fireVector[];
	unsigned int& warpResult = fireVector[threadIdx.x];
	unsigned int* resultVectorBase = fireVector + warp::index() * 32;
	unsigned int warpThread = warp::thread();
	unsigned int limit = numNeurons + 31;
	unsigned int mask = bit::mask(warpThread);
	while (index < limit)
	{
		warpResult = 0;
		if (index < numNeurons)
		{
			float _delay = delay[index];
			const float _interval = interval[index];
			_delay -= dt;
			if (_delay < 0.0f)
			{
				warpResult = mask;
				_delay += _interval;
			}
			delay[index] = _delay;
			neuronVoltageVector[index] = _delay;
		}
		warp::reduceOr(resultVectorBase, warpThread);
		if (warp::leader())
		{
			neuronFireVector[bit::word(index)] = warpResult;
		}
		index += stride;
	}
}

void pulse(float* delay,
           const float* interval,
		   unsigned int* neuronFireVector,
		   float* neuronVoltageVector,
		   float dt,
		   unsigned int numNeurons)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid =
		(numNeurons + threadsPerBlock - 1) / threadsPerBlock;
	size_t shmemSize = threadsPerBlock * sizeof(unsigned int);
	pulseKernel<<<blocksPerGrid, 
	              threadsPerBlock, 
				  shmemSize, 
				  slug::sim::CUDA::stream>>>
		(delay,
		 interval,
		 neuronFireVector,
		 neuronVoltageVector,
		 dt,
		 numNeurons);
}

