#include <slug/sim/NeuronSimulator.h>
#include <slug/sim/CUDA.h>
#include "CUDA.h"

using namespace slug;
using namespace slug::sim;

class CUDAPulseSimulator : public sim::NeuronSimulator<util::MemoryType::CUDA>
{
public:
	static const auto CUDA = util::MemoryType::CUDA;
	static const auto System = util::MemoryType::System;
	CUDAPulseSimulator()
	{
	}

	virtual bool init(const SimulationProperties* properties)
	{
		_properties = properties;
		_timestep = float(_properties->timestep) / 1000.0f;

		require("delay");
		require("interval");
		
		if (!validate())
			return false;

		_delay = _allocate<float>(numNeurons);
		_transfer<CUDA, System>
		    (_delay,
			 getParameterArray<float>("delay"),
			 numNeurons);

		_interval = _allocate<float>(numNeurons);
		_transfer<CUDA, System>
		    (_interval,
			 getParameterArray<float>("interval"),
			 numNeurons);

		return true;
	}

	virtual void initVoltages(float* voltages,
	                          const SimulationProperties* properties)
	{
		_zero<float>(voltages, numNeurons);
	}

	virtual void update(UpdateParameters& parameters)
	{
		pulse(_delay,
		      _interval,
			  parameters.neuronFireVector,
			  parameters.neuronVoltage,
			  _timestep,
			  numNeurons);
		slug::sim::CUDA::synchronize();
	}

private:
	const SimulationProperties* _properties;
	float _timestep;
	float* _delay;
	float* _interval;
};

extern "C"
{

void activate()
{
	{
		static const auto type = util::MemoryType::CUDA;
		auto sim =
			new sim::SpecificFactory<sim::NeuronSimulator<type>,
			                         CUDAPulseSimulator>("pulse", "pulse", 0);
		sim::Manager<sim::Factory<sim::NeuronSimulator<type>>>::add(sim);
	}
}

}
