#include "CUDA.h"
#include <slug/util/CUDA.h>
#include <slug/sim/CUDA.h>
#include <slug/util/Bit.h>
#include <iostream>
#include <stdio.h>

template<typename T>
__global__ void extractKernel(typename Storage<T>::elementType* source,
                              typename Storage<T>::elementType* destination,
							  unsigned int* indices,
							  unsigned int numIndices)
{
	unsigned int index = blockDim.x * blockIdx.x + threadIdx.x;
	unsigned int stride = blockDim.x * gridDim.x;
	while (index < numIndices)
	{
		destination[index] = source[indices[index]];
		index += stride;
	}
}

//Bit specialization
template<>
__global__ void extractKernel<Bit>(unsigned int* source,
                                   unsigned int* destination,
								   unsigned int* indices,
								   unsigned int numIndices)
{
	unsigned int index = blockDim.x * blockIdx.x + threadIdx.x;
	unsigned int stride = blockDim.x * gridDim.x;
	extern __shared__ unsigned int resultVector[];
	unsigned int& warpResult = resultVector[threadIdx.x];
	unsigned int* resultVectorBase = resultVector + 32 * warp::index();
	unsigned int warpThread = warp::thread();
	unsigned int limit = ((numIndices + 31) / 32) * 32;
	while (index < limit)
	{
		warpResult = 0;
		if (index < numIndices)
		{
			unsigned int i = indices[index];
			unsigned int bit = bit::bit(i);
			unsigned int word = source[bit::word(i)];
			printf("%u %u %x\n", i, bit, word);
			warpResult = bit::extract(word, bit);
			warpResult >>= warpThread;
		}
		warp::reduceOr(resultVectorBase, warpThread);
		if (warp::leader())
			destination[bit::word(index)] = warpResult;

		index += stride;
	}
}

template<typename T>
void extract(typename Storage<T>::elementType* d_source,
             typename Storage<T>::elementType* d_destination,
			 unsigned int* d_indices,
			 unsigned int numIndices)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid = 
		(numIndices + threadsPerBlock - 1) / threadsPerBlock;
	extractKernel<T><<<blocksPerGrid, 
	                   threadsPerBlock, 
					   0, 
					   slug::sim::CUDA::stream>>>
		(d_source, d_destination, d_indices, numIndices);
}

template<>
void extract<Bit>(Storage<Bit>::elementType* d_source,
                  Storage<Bit>::elementType* d_destination,
				  unsigned int* d_indices,
				  unsigned int numIndices)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid = 
		(numIndices + threadsPerBlock - 1) / threadsPerBlock;
	unsigned int shmemSize = threadsPerBlock * sizeof(unsigned int);
	extractKernel<Bit><<<blocksPerGrid, 
	                   threadsPerBlock, 
					   shmemSize, 
					   slug::sim::CUDA::stream>>>
		(d_source, d_destination, d_indices, numIndices);
}

template void extract<int>(int*, int*, unsigned int*, unsigned int);
template void extract<float>(float*, float*, unsigned int*, unsigned int);
template void extract<Bit>(unsigned int*, unsigned int*, 
                           unsigned int*, unsigned int);
