#include <slug/sim/Reportable.h>
#include <slug/sim/SynapseSimulator.h>

template<slug::util::MemoryType MType>
class FiringBuffer
	: public slug::sim::DataBuffer,
	  public slug::util::Memory<MType>
{
public:
	FiringBuffer(unsigned int _capacity, unsigned int numSynapses);
	void expand(unsigned int _capacity);
	unsigned int* deviceNumIndices;
	unsigned int numIndices;
	unsigned int* firingIndices;
	float* firingValues;
	unsigned int* PSGCounts;
	unsigned int capacity;

	float* USE;
private:
};

template<slug::util::MemoryType MType>
class NCS5Simulator
	: public slug::sim::SynapseSimulator<MType>,
	  public slug::sim::SpecificPublisher<FiringBuffer<MType>>
{
public:
	virtual bool init(const slug::sim::SimulationProperties* properties);

	virtual 
	void initWeights(float* weights,
	                 const slug::sim::SimulationProperties* properties);

	typedef typename slug::sim::SynapseSimulator<MType>::UpdateParameters
		UpdateParameters;

	virtual void update(UpdateParameters& parameters);
private:
	template<typename T> T* loadParameter(const std::string& name);

	unsigned char* _RSEType;
	float* _USEBase;
	//float* _USE;
	float* _RSE;
	float* _maxG;
	float* _tauDepression;
	float* _tauFacilitation;
	bool* _learningOn;
	unsigned char* _learningType;
	float* _positiveLearningModulator;
	float* _positivePeakTime;
	float* _positivePeakDeltaUSE;
	float* _positiveWindowWidth;
	float* _negativeLearningModulator;
	float* _negativePeakTime;
	float* _negativePeakDeltaUSE;
	float* _negativeWindowWidth;

	float* _synapticReversal;
	const float** _PSGs;
	float* _PSGData;
	unsigned int* _PSGLengths;
	float* _lastPreFireTime;
	float* _lastPostFireTime;

	float _elapsedTime;
	float _timestep;

	slug::sim::Subscription<FiringBuffer<MType>>* _firingBufferSubscription;
};

#include <NCS5.hpp>
