#include <NCS5.h>
#include <CUDA.cuh>
#include <slug/sim/CUDA.h>

template<>
void NCS5Simulator<slug::util::MemoryType::CUDA>::
update(UpdateParameters& parameters)
{
	const auto CUDA = slug::util::MemoryType::CUDA;
	const auto System = slug::util::MemoryType::System;
	_elapsedTime += _timestep;

	auto oldBuffer = _firingBufferSubscription->pull();
	auto newBuffer = _getBlank();

	_transfer<System, CUDA>(&oldBuffer->numIndices, 
	                        oldBuffer->deviceNumIndices, 1);
	slug::sim::CUDA::synchronize();
	unsigned int newCapacity = oldBuffer->numIndices + 2 * numSynapses;
	//oldBuffer->expand(oldBuffer->numIndices + 2 * numSynapses);
	_zero<unsigned int>(newBuffer->deviceNumIndices, 1);
	newBuffer->numIndices = 0;
	newBuffer->expand(newCapacity);
	
	_transfer<CUDA, CUDA>(newBuffer->USE, oldBuffer->USE, numSynapses);

#if 1
	//Get the new firings and stack them to the end of the old buffer
	receiveFirings(_RSEType,
	               _USEBase,
				   newBuffer->USE,
				   _RSE,
				   _maxG,
				   _tauDepression,
				   _tauFacilitation,
				   _lastPreFireTime,
				   _lastPostFireTime,
				   _learningOn,
				   _learningType,
				   _positiveLearningModulator,
				   _positivePeakTime,
				   _positivePeakDeltaUSE,
				   _negativeWindowWidth,
				   _negativePeakTime,
				   _negativePeakDeltaUSE,
				   _negativeLearningModulator,
				   _PSGLengths,
				   oldBuffer->firingValues,
				   oldBuffer->firingIndices,
				   oldBuffer->PSGCounts,
				   oldBuffer->deviceNumIndices,
				   _elapsedTime,
				   parameters.synapticFireVector,
				   numSynapses);
#endif

#if 1
	//Do any positive learning
	computePostFirings(parameters.neuronFireVector,
	                   parameters.neuronFireVectorSize,
					   postsynapticNeuronIDs,
					   _USEBase,
					   _lastPreFireTime,
					   _lastPostFireTime,
					   _negativeLearningModulator,
					   _negativePeakTime,
					   _negativePeakDeltaUSE,
					   _positiveLearningModulator,
					   _positiveWindowWidth,
					   _positivePeakTime,
					   _positivePeakDeltaUSE,
					   _learningType,
					   _learningOn,
					   _elapsedTime,
					   numSynapses);
#endif

#if 1
	//Get the synaptic contributions of all firings
	computeSynapticWeights(oldBuffer->firingValues,
	                       oldBuffer->firingIndices,
						   oldBuffer->PSGCounts,
						   _PSGs,
						   _synapticReversal,
						   postsynapticNeuronIDs,
						   parameters.neuronVoltageVector,
						   parameters.synapticCurrentVector,
						   oldBuffer->deviceNumIndices,
						   newBuffer->firingIndices,
						   newBuffer->PSGCounts,
						   newBuffer->firingValues,
						   newBuffer->deviceNumIndices);
#endif
	slug::sim::CUDA::synchronize();
	publish(newBuffer);
	oldBuffer->release();
}

template<>
void NCS5Simulator<slug::util::MemoryType::System>::
update(UpdateParameters& parameters)
{
}
