#include <slug/util/CUDA.h>
#include <slug/sim/CUDA.h>
#include "CUDA.h"
#include <algorithm>

__global__ void stdpKernel(const unsigned int* postNeuronID,
                           const unsigned int* neuronFireVector,
						   const unsigned int neuronFireVectorSize,
                           const unsigned int* presynapticFireVector,
						   float* synapticCurrent,
						   float* synapticWeight,
						   const float* APlus,
						   const float* AMinus,
						   const float* tauPlus,
						   const float* tauMinus,
						   float* presynapticFireTime,
						   float* postsynapticFireTime,
						   const float elapsedTime,
						   const unsigned int numSynapses)
{
	//Load the neuron fire vector into shared memory
	#if 0
	extern __shared__ unsigned int _neuronFireVector[];
	unsigned int blockStride = block::stride();
	for (unsigned int i = block::thread(); 
	     i < neuronFireVectorSize; 
		 i += blockStride)
	{
		_neuronFireVector[i] = neuronFireVector[i];
	}
	__syncthreads();
	#endif
	unsigned int gridStride = grid::stride();
	for (unsigned int i = grid::thread(); i < numSynapses; i += gridStride)
	{
		float dWeight = 0.0f;
		float _preFireTime = presynapticFireTime[i];
		float _postFireTime = postsynapticFireTime[i];

		bool fired = false;
		bool changed = false;
		//Presynaptic firing
		{
			unsigned int word = presynapticFireVector[bit::word(i)];
			if (bit::extract(word, bit::bit(i)) != 0)
			{
				presynapticFireTime[i] = elapsedTime;
				dWeight -= AMinus[i] * 
				           exp((_postFireTime - elapsedTime) / tauMinus[i]);
				fired = true;
				changed = true;
			}
		}
		
		unsigned int postID = postNeuronID[i];
		//Postsynaptic firing
		{
			unsigned int word = neuronFireVector[bit::word(postID)];
			if (bit::extract(word, bit::bit(postID)) != 0)
			{
				postsynapticFireTime[i] = elapsedTime;
				dWeight += APlus[i] *
				           exp((_preFireTime - elapsedTime) / tauPlus[i]);
				changed = true;
			}
		}
		
		//Add the result
		float weight = dWeight;
		if (changed)
		{
			weight += synapticWeight[i];
			synapticWeight[i] = weight;
		}
		if (fired)
		{
			atomicAdd(synapticCurrent + postID, weight);
		}
	}
}

void stdp(const unsigned int* postNeuronID,
          const unsigned int* neuronFireVector,
		  const unsigned int neuronFireVectorSize,
		  const unsigned int* presynapticFireVector,
		  float* synapticCurrent,
		  float* synapticWeight,
		  const float* APlus,
		  const float* AMinus,
		  const float* tauPlus,
		  const float* tauMinus,
		  float* presynapticFireTime,
		  float* postsynapticFireTime,
		  const float elapsedTime,
		  const unsigned int numSynapses)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid =
		(numSynapses + threadsPerBlock - 1) / threadsPerBlock;
	blocksPerGrid = std::min(256u, blocksPerGrid);
	size_t shmemSize = 0;
	//shmemSize += sizeof(unsigned int) * neuronFireVectorSize;
	stdpKernel<<<blocksPerGrid, 
	             threadsPerBlock, 
				 shmemSize, 
				 slug::sim::CUDA::stream>>>
		(postNeuronID,
		 neuronFireVector,
		 neuronFireVectorSize,
		 presynapticFireVector,
		 synapticCurrent,
		 synapticWeight,
		 APlus,
		 AMinus,
		 tauPlus,
		 tauMinus,
		 presynapticFireTime,
		 postsynapticFireTime,
		 elapsedTime,
		 numSynapses);
}

