%{
#include <fstream>
#include <string>
#include <stack>
#include <algorithm>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include <iostream>
#include <slug/readers/ncs5/File.h>

extern int yylex();
extern long lineNumber;
extern std::string currentFilename;
extern std::string currentPath;
extern FILE* yyin;

using namespace slug::readers::ncs5::file;

NCSFile* ncsFile = nullptr;
Brain* brain = nullptr;
ColumnShell* columnShell = nullptr;
Column* column = nullptr;
LayerShell* layerShell = nullptr;
Layer* layer = nullptr;
Cell* cell = nullptr;
Compartment* compartment = nullptr;
Channel* channel = nullptr;
KmChannel* kmChannel = nullptr;
KahpChannel* kahpChannel = nullptr;
KaChannel* kaChannel = nullptr;
enum CurrentChannel
{
	None,
	Km,
	Kahp,
	Ka
};
CurrentChannel currentChannel;
Synapse* synapse = nullptr;
RSE* rse = nullptr;
Learning* learning = nullptr;
PSG* psg = nullptr;
SynapticAugmentation* augmentation = nullptr;
SpikeShape* spikeShape = nullptr;
Stimulus* stimulus = nullptr;
StimulusInject* injection = nullptr;
Report* report = nullptr;
Event* event = nullptr;

std::stack<std::string> filenameStack;
std::stack<long> lineNumberStack;
std::stack<FILE*> fileStack;
std::stack<std::string> pathStack;

bool outputParserDebug = true;

void debugp(const char* message, ...)
{
	if (outputParserDebug)
	{
		va_list args;
		va_start(args, message);
		vfprintf(stdout, message, args);
		va_end(args);
	}
}

void yyerror(const char* error)
{
	fprintf(stderr, "File: %s: %ld: %s\n", 
	        currentFilename.c_str(), lineNumber, error);
	exit(-1);
}

%}

%defines
%union
{
	std::vector<float>* numberList;
	int integer;
	float floatingPoint;
	std::string* string;
	struct NormalDistribution* distribution;
}

%start file
%token <integer> INTEGER
%token <floatingPoint> FLOAT
%token <string> STRING
%type <floatingPoint> NUMBER
%type <numberList> NUMBERLIST
%type <distribution> DISTRIBUTION
%token ABSOLUTE_USE
%token ALPHA
%token ALPHA_SCALE_FACTOR_H
%token ALPHA_SCALE_FACTOR_M
%token AMP_END
%token AMP_START
%token ASCII
%token AUTO
%token AUGMENTATION_DELAY
%token AUGMENTATION_INIT
%token AUGMENTATION_TAU
%token AVERAGE_SYN
%token BETA_SCALE_FACTOR_H
%token BETA_SCALE_FACTOR_M
%token BOTH
%token BRAIN
%token CA_EXP_FACTOR
%token CA_EXP_GAUSS
%token CA_EXP_RANGE
%token CA_EXTERNAL
%token CA_HALF_MIN
%token CA_INTERNAL
%token CA_SCALE_FACTOR
%token CA_SPIKE_INCREMENT
%token CA_TAU
%token CA_TAU_DECAY
%token CA_TAU_SCALE_FACTOR
%token CELL
%token CELL_SEQUENCE
%token CELLS
%token CELL_TYPE
%token CHANNEL
%token COLUMN
%token COLUMN_SHELL
%token COLUMN_TYPE
%token COMPARTMENT
%token MAX_CONDUCT
%token CONNECT
%token DATA_LABEL
%token DELAY
%token DEPR
%token DEPR_TAU
%token DISTANCE
%token DURATION
%token END_BRAIN
%token END_CELL
%token END_CHANNEL
%token END_COLUMN
%token END_COLUMN_SHELL
%token END_COMPARTMENT
%token END_EVENT
%token END_LAYER
%token END_LAYER_SHELL
%token END_REPORT
%token END_SPIKESHAPE
%token END_STIMULUS
%token END_STIMULUS_INJECT
%token END_SYNAPSE
%token END_SYN_AUGMENTATION
%token END_SYN_DATA
%token END_SYN_FACIL_DEPRESS
%token END_SYN_LEARNING
%token END_SYN_PSG
%token EXP
%token EXPONENT
%token EVENT
%token E_HALF_MIN_H
%token E_HALF_MIN_M
%token FACIL
%token FACIL_TAU
%token FILENAME
%token FREQUENCY
%token FREQ_ROWS
%token FREQ_COLS
%token FREQ_START
%token FSV
%token HEBB_ON
%token HEBB_OFF
%token HEBB_START
%token HEBB_END
%token HEIGHT
%token HIDE_TIMESTEP
%token H_INITIAL
%token H_POWER
%token IGNORE_EMPTY
%token INCLUDE
%token INJECT
%token INTERACTIVE
%token LAYER
%token LAYER_SHELL
%token LAYER_TYPE
%token LEAK_CONDUCTANCE
%token LEAK_REVERSAL
%token LEARNING
%token LEARNING_SHAPE
%token LEARN_LABEL
%token LOAD
%token LOCATION
%token LOWER
%token MAX_AUGMENTATION
%token MAX_G
%token MODE
%token M_INITIAL
%token M_POWER
%token NEG_HEBBIAN
%token NEG_HEB_PEAK_DELTA_USE
%token NEG_HEB_PEAK_TIME
%token NEG_HEB_WINDOW
%token NONE
%token OUTPUT_CELLS
%token OUTPUT_CONNECT_MAP
%token OPTION
%token PATTERN
%token PHASE
%token PHASE_SHIFT
%token POS_HEBBIAN
%token POS_HEB_PEAK_DELTA_USE
%token POS_HEB_PEAK_TIME
%token POS_HEB_WINDOW
%token PREV_SPIKE_RANGE
%token PROB
%token RECURRENT_CONNECT
%token RELOAD_SYN
%token REPORT
%token REPORT_ON
%token REVERSAL_POTENTIAL
%token RSE
%token RSE_LABEL
%token RSE_INIT
%token R_MEMBRANE
%token SAMESEED
%token SAVE
%token SAVE_SYN
%token SEED
%token SINGLE
%token SELECT_FRONT
%token SERVER
%token SERVER_PORT
%token SLOPE_FACTOR_H
%token SLOPE_FACTOR_H_STDEV
%token SLOPE_FACTOR_M
%token SLOPE_FACTOR_M_STDEV
%token SPIKESHAPE
%token SPIKE_HALFWIDTH
%token STIMULUS
%token STIMULUS_INJECT
%token STIM_TYPE
%token STRENGTH
%token STRENGTH_GAUSS
%token STRENGTH_RANGE
%token SYNAPSE
%token SYNAPSE_RSE
%token SYNAPSE_UF
%token SYN_AUGMENTATION
%token SYN_CALCIUM
%token SYN_DATA
%token SYN_FACIL_DEPRESS
%token SYN_LEARNING
%token SYN_PSG
%token SYN_REVERSAL
%token TAU_MEMBRANE
%token TAU_SCALE_FACTOR_H
%token TAU_SCALE_FACTOR_M
%token THRESHOLD
%token TIME_END
%token TIME_START
%token TIMING
%token TRIANGLE
%token TYPE
%token UNITARY_G
%token UPPER
%token USE_OVERRIDE
%token VERT_TRANS
%token VMREST
%token VOLTAGES
%token V_TAU_VALUE_H
%token V_TAU_VOLTAGE_H
%token V_TAU_VALUE_M
%token V_TAU_VOLTAGE_M
%token V_TAU_VALUE_H_STDEV
%token V_TAU_VOLTAGE_H_STDEV
%token V_TAU_VALUE_M_STDEV
%token V_TAU_VOLTAGE_M_STDEV
%token VERSION
%token WARNINGS_OFF
%token WIDTH
%token SFD
%token SFD_LABEL
%token PSG_FILE
%token CELLS_PER_FREQ
%token TIME_INCREMENT
%token DYN_RANGE
%token PORT
%token JOB
%token DISTRIBUTE
%token CONNECT_REPORT
%token SPIKE_REPORT
%token RATE
%token TAU_NOISE
%token CORREL
%token NO
%token YES

%%

file : object_specification_list
{
}

object_specification_list : object_specification_list object_specification 
{
}
| object_specification
{
}

object_specification : brain_specification
{
	if (!brain->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}

	if (ncsFile->brains.count(brain->type))
	{
		std::cout << "Brain " << brain->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->brains[brain->type] = brain;
}
| column_specification
{
	if (!column->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->columns.count(column->type))
	{
		std::cout << "Column " << column->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->columns[column->type] = column;
}
| column_shell_specification
{
	if (!columnShell->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->columnShells.count(columnShell->type))
	{
		std::cout << "ColumnShell " << columnShell->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->columnShells[columnShell->type] = columnShell;
}
| layer_specification
{
	if (!layer->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->layers.count(layer->type))
	{
		std::cout << "Layer " << layer->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->layers[layer->type] = layer;
}
| layer_shell_specification
{
	if (!layerShell->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->layerShells.count(layerShell->type))
	{
		std::cout << "LayerShell " << layerShell->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->layerShells[layerShell->type] = layerShell;
}
| cell_specification
{
	if (!cell->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->cells.count(cell->type))
	{
		std::cout << "Cell " << cell->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->cells[cell->type] = cell;
}
| compartment_specification
{
	if (!compartment->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->compartments.count(compartment->type))
	{
		std::cout << "Compartment " << compartment->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->compartments[compartment->type] = compartment;
}
| channel_specification
{
	if (!channel->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->channels.count(channel->type))
	{
		std::cout << "Channel " << channel->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->channels[channel->type] = channel;
}
| synapse_specification
{
	if (!synapse->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->synapses.count(synapse->type))
	{
		std::cout << "Synapse " << synapse->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->synapses[synapse->type] = synapse;
}
| sfd_specification
{
	if (!rse->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->RSEs.count(rse->type))
	{
		std::cout << "RSE " << rse->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->RSEs[rse->type] = rse;
}
| learning_specification
{
	if (!learning->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->learnings.count(learning->type))
	{
		std::cout << "Learning " << learning->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->learnings[learning->type] = learning;
}
| psg_specification
{
	if (!psg->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->PSGs.count(psg->type))
	{
		std::cout << "PSG " << psg->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->PSGs[psg->type] = psg;
}
| augmentation_specification
{
	if (!augmentation->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->augmentations.count(augmentation->type))
	{
		std::cout << "Augmentation " << augmentation->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->augmentations[augmentation->type] = augmentation;
}
| spikeshape_specification
{
	if (!spikeShape->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->spikeShapes.count(spikeShape->type))
	{
		std::cout << "SpikeShape " << spikeShape->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->spikeShapes[spikeShape->type] = spikeShape;
}
| stimulus_specification
{
	if (!stimulus->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->stimuli.count(stimulus->type))
	{
		std::cout << "SpikeShape " << stimulus->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->stimuli[stimulus->type] = stimulus;
}
| stimulus_inject_specification
{
	if (!injection->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->injections.count(injection->type))
	{
		std::cout << "Injection " << injection->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->injections[injection->type] = injection;
}
| report_specification
{
	if (!report->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->reports.count(report->type))
	{
		std::cout << "Report " << report->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->reports[report->type] = report;
}
| event_specification
{
	if (!event->validate())
	{
		yyerror("Validation failed");
		exit(-1);
	}
	if (ncsFile->events.count(event->type))
	{
		std::cout << "Event " << event->type.value << 
		    " redefined." << std::endl;
	}
	ncsFile->events[event->type] = event;
}
| INCLUDE STRING
{
	std::string file = *$2;
	if (file[0] != '/')
	{
		file = currentPath + "/" + file;
	}
	FILE* fp = fopen(file.c_str(), "r");
	if (!fp)
	{
		printf("Could not open include file %s\n", file.c_str());
		exit(-1);
	}
	else
	{
		fileStack.push(yyin);
		filenameStack.push(currentFilename);
		lineNumberStack.push(lineNumber);
		pathStack.push(currentPath);

		std::string newPath;
		const auto lastSlashPosition = file.rfind('/');
		if (lastSlashPosition != std::string::npos)
		{
			newPath = file.substr(0, lastSlashPosition);
		}
		else
		{
			newPath = "./";
		}

		yyin = fp;
		currentFilename = *$2;
		lineNumber = 1;
		currentPath = newPath;
	}
	delete $2;
}

brain_specification : BRAIN { brain = new Brain(); } brain_parameter_list END_BRAIN
{
}

brain_parameter_list : brain_parameter brain_parameter_list
{
}
|
{
}

brain_parameter : TYPE STRING
{
	brain->type= *$2;
	delete $2;
}
| JOB STRING
{
	brain->job = *$2;
	delete $2;
}
| DURATION NUMBER
{
	brain->duration = $2;
}
| FSV INTEGER
{
	brain->frequencySamplingValue = $2;
}
| SERVER_PORT INTEGER
{
	brain->port = $2;
}
| SERVER STRING
{
	brain->server = *$2;
	delete $2;
}
| PORT STRING
{
}
| PORT AUTO
{
}
| INTERACTIVE YES
{
	brain->interactive = true;
}
| INTERACTIVE NO
{
	brain->interactive = false;
}
| SEED INTEGER
{
	brain->seed = $2;
}
| DISTANCE YES
{
	brain->distance = true;
}
| DISTANCE NO
{
	brain->distance = false;
}
| COLUMN_TYPE STRING
{
	brain->columns.push_back(*$2);
	delete $2;
}
| STIMULUS_INJECT STRING
{
	brain->stimuli.push_back(*$2);
	delete $2;
}
| EVENT STRING
{
	brain->events.push_back(*$2);
	delete $2;
}
| REPORT STRING
{
	brain->reports.push_back(*$2);
	delete $2;
}
| CONNECT STRING STRING STRING STRING STRING STRING STRING STRING STRING NUMBER NUMBER
{
	Brain::Connection* bc = new Brain::Connection();
	bc->source.column = *$2;
	bc->source.layer = *$3;
	bc->source.cellType = *$4;
	bc->source.compartment = *$5;
	bc->destination.column = *$6;
	bc->destination.layer = *$7;
	bc->destination.cellType = *$8;
	bc->destination.compartment = *$9;
	bc->synapseType = *$10;
	bc->probability = $11;
	bc->speed = $12;
	bc->decayStep = 0;
	brain->connections.push_back(bc);
	delete $2;
	delete $3;
	delete $4;
	delete $5;
	delete $6;
	delete $7;
	delete $9;
	delete $10;
}
| CONNECT STRING STRING STRING STRING STRING STRING STRING STRING STRING NUMBER NUMBER NUMBER 
{
	Brain::Connection* bc = new Brain::Connection();
	bc->source.column = *$2;
	bc->source.layer = *$3;
	bc->source.cellType = *$4;
	bc->source.compartment = *$5;
	bc->destination.column = *$6;
	bc->destination.layer = *$7;
	bc->destination.cellType = *$8;
	bc->destination.compartment = *$9;
	bc->synapseType = *$10;
	bc->probability = $11;
	bc->speed = $12;
	bc->decayStep = $13;
	brain->connections.push_back(bc);
	delete $2;
	delete $3;
	delete $4;
	delete $5;
	delete $6;
	delete $7;
	delete $9;
	delete $10;
}
| RECURRENT_CONNECT STRING STRING STRING STRING STRING STRING STRING STRING STRING NUMBER NUMBER
{
	Brain::Connection* bc = new Brain::Connection();
	bc->source.column = *$2;
	bc->source.layer = *$3;
	bc->source.cellType = *$4;
	bc->source.compartment = *$5;
	bc->destination.column = *$6;
	bc->destination.layer = *$7;
	bc->destination.cellType = *$8;
	bc->destination.compartment = *$9;
	bc->synapseType = *$10;
	bc->probability = $11;
	bc->speed = $12;
	bc->decayStep = 0;
	brain->connections.push_back(bc);
	delete $2;
	delete $3;
	delete $4;
	delete $5;
	delete $6;
	delete $7;
	delete $9;
	delete $10;

	Brain::Connection* rbc = new Brain::Connection();
	*rbc = *bc;
	std::swap(rbc->source, rbc->destination);
}
| SAVE STRING FLOAT
{
	Brain::Save* save = new Brain::Save();
	save->filename = *$2;
	save->time = $3;
	brain->saves.push_back(save);
	delete $2;
}
| LOAD STRING
{
	Brain::Load* load = new Brain::Load();
	load->filename = *$2;
	brain->loads.push_back(load);
	delete $2;
}
| OPTION WARNINGS_OFF
{
	brain->warnings = false;
}
| OUTPUT_CELLS YES
{
	brain->outputCells = true;
}
| OUTPUT_CELLS NO
{
	brain->outputCells = false;
}
| OUTPUT_CONNECT_MAP YES
{
	brain->outputConnectionMap = true;
}
| OUTPUT_CONNECT_MAP NO
{
	brain->outputConnectionMap = false;
}

column_shell_specification : COLUMN_SHELL { columnShell = new ColumnShell(); } column_shell_parameter_list END_COLUMN_SHELL
{
}

column_shell_parameter_list : column_shell_parameter column_shell_parameter_list
{
}
|
{
}

column_shell_parameter : TYPE STRING
{
	columnShell->type = *$2;
	delete $2;
}
| WIDTH NUMBER
{
	columnShell->width = $2;
}
| HEIGHT NUMBER
{
	columnShell->height = $2;
}
| LOCATION NUMBER NUMBER
{
	columnShell->x = $2;
	columnShell->y = $3;
}

column_specification : COLUMN { column = new Column(); } column_parameter_list END_COLUMN
{
}

column_parameter_list : column_parameter column_parameter_list
{
}
|
{
}

column_parameter : TYPE STRING
{
	column->type = *$2;
	delete $2;
} 
| COLUMN_SHELL STRING
{
	column->columnShell = *$2;
	delete $2;
}
| LAYER_TYPE STRING
{
	column->layers.push_back(*$2);
	delete $2;
}
| CONNECT STRING STRING STRING STRING STRING STRING STRING NUMBER NUMBER
{
	Column::Connection* cc = new Column::Connection();
	cc->source.layer = *$2;
	cc->source.cellType = *$3;
	cc->source.compartment = *$4;
	cc->destination.layer = *$5;
	cc->destination.cellType = *$6;
	cc->destination.compartment = *$7;
	cc->synapseType = *$8;
	cc->probability = $9;
	cc->speed = $10;
	delete $2;
	delete $3;
	delete $4;
	delete $5;
	delete $6;
	delete $7;
	delete $8;
	column->connections.push_back(cc);
}

layer_shell_specification : LAYER_SHELL { layerShell = new LayerShell(); } layer_shell_parameter_list END_LAYER_SHELL
{
}

layer_shell_parameter_list : layer_shell_parameter layer_shell_parameter_list
{
}
|
{
}

layer_shell_parameter : TYPE STRING
{
	layerShell->type = *$2;
	delete $2;
}
| LOWER NUMBER
{
	layerShell->lower = $2;
}
| UPPER NUMBER
{
	layerShell->upper = $2;
}

layer_specification : LAYER { layer = new Layer(); } layer_parameter_list END_LAYER
{
}

layer_parameter_list : layer_parameter layer_parameter_list
{
}
|
{
}

layer_parameter : TYPE STRING
{
	layer->type = *$2;
	delete $2;
}
| LAYER_SHELL STRING
{
	layer->layerShell = *$2;
	delete $2;
}
| CELL_TYPE STRING INTEGER
{
	Layer::CellType* ct = new Layer::CellType();
	ct->type = *$2;
	ct->count = $3;
	layer->cellTypes.push_back(ct);
	delete $2;
}
| CONNECT STRING STRING STRING STRING STRING NUMBER INTEGER
{
	Layer::Connection* lc = new Layer::Connection();
	lc->source.cellType = *$2;
	lc->source.compartment = *$3;
	lc->destination.cellType = *$4;
	lc->destination.compartment = *$5;
	lc->synapseType = *$6;
	lc->probability = $7;
	lc->speed = $8;
	layer->connections.push_back(lc);
	delete $2;
	delete $3;
	delete $4;
	delete $5;
	delete $6;
}

cell_specification : CELL { cell = new Cell(); } cell_parameter_list END_CELL
{
}

cell_parameter_list : cell_parameter cell_parameter_list
{
}
|
{
}

cell_parameter : TYPE STRING
{
	cell->type = *$2;
	delete $2;
}
| COMPARTMENT STRING STRING NUMBER NUMBER NUMBER
{
	Cell::Compartment* c = new Cell::Compartment();
	c->name = *$2;
	c->label = *$3;
	cell->compartments.push_back(c);
	delete $2;
	delete $3;
}
| CONNECT STRING STRING INTEGER NUMBER
{
	Cell::Connection* cc = new Cell::Connection();
	cc->source = *$2;
	cc->destination = *$3;
	cc->speed = $4;
	cc->G = $5;
	cell->connections.push_back(cc);
	delete $2;
	delete $3;
}

compartment_specification : COMPARTMENT { compartment = new Compartment(); } compartment_parameter_list END_COMPARTMENT
{
}

compartment_parameter_list : compartment_parameter compartment_parameter_list
{
}
|
{
}

compartment_parameter : TYPE STRING
{
	compartment->type = *$2;
	delete $2;
}
| SEED INTEGER
{
	compartment->seed = $2;
}
| SPIKESHAPE STRING
{
	compartment->spikeShape = *$2;
	delete $2;
}
| SPIKE_HALFWIDTH DISTRIBUTION
{
	//Deprecated
}
| TAU_MEMBRANE DISTRIBUTION
{
	compartment->tauMembrane = *$2;
	delete $2;
}
| R_MEMBRANE DISTRIBUTION
{
	compartment->rMembrane = *$2;
	delete $2;
}
| THRESHOLD DISTRIBUTION
{
	compartment->threshold = *$2;
	delete $2;
}
| LEAK_REVERSAL DISTRIBUTION
{
	compartment->leakReversal = *$2;
	delete $2;
}
| LEAK_CONDUCTANCE DISTRIBUTION
{
	compartment->leakConductance = *$2;
	delete $2;
}
| VMREST DISTRIBUTION
{
	compartment->vmRest = *$2;
	delete $2;
}
| CA_INTERNAL DISTRIBUTION
{
	compartment->caInternal = *$2;
	delete $2;
}
| CA_EXTERNAL DISTRIBUTION
{
	compartment->caExternal = *$2;
	delete $2;
}
| CA_SPIKE_INCREMENT DISTRIBUTION
{
	compartment->caSpikeIncrement = *$2;
	delete $2;
}
| CA_TAU DISTRIBUTION
{
	compartment->caTau = *$2;
	delete $2;
}
| CHANNEL STRING
{
	compartment->channels.push_back(*$2);
	delete $2;
}

channel_specification : CHANNEL STRING 
{
	if (*$2 == "Km")
	{
		kmChannel = new KmChannel();
		channel = kmChannel;
		currentChannel = Km;
	}
	else if (*$2 == "Ka")
	{
		kaChannel = new KaChannel();
		channel = kaChannel;
		currentChannel = Ka;
	}
	else if (*$2 == "Kahp")
	{
		kahpChannel = new KahpChannel();
		channel = kahpChannel;
		currentChannel = Kahp;
	}
	delete $2;
} channel_parameter_list END_CHANNEL
{
}

channel_parameter_list : channel_parameter channel_parameter_list
{
}
|
{
}

channel_parameter : TYPE STRING
{
	channel->type = *$2;
	delete $2;
}
| M_INITIAL DISTRIBUTION
{
	channel->mInitial = *$2;
	delete $2;
}
| M_POWER DISTRIBUTION
{
	channel->mPower = *$2;
	delete $2;
}
| REVERSAL_POTENTIAL DISTRIBUTION 
{
	channel->reversalPotential = *$2;
	delete $2;
}
| UNITARY_G DISTRIBUTION
{
	channel->unitaryG = *$2;
	delete $2;
}
| STRENGTH DISTRIBUTION
{
	channel->strength = *$2;
	delete $2;
}
| SLOPE_FACTOR_M NUMBER NUMBER NUMBER
{
	kmChannel->slopeFactorM[0] = $2;
	kmChannel->slopeFactorM[1] = $3;
	kmChannel->slopeFactorM[2] = $4;
}
| TAU_SCALE_FACTOR_M DISTRIBUTION
{
	kmChannel->tauScaleFactorM = *$2;
	delete $2;
}
| SEED INTEGER
{
	channel->seed = $2;
}
| CA_SCALE_FACTOR DISTRIBUTION
{
	kahpChannel->caScaleFactor = *$2;
	delete $2;
}
| CA_EXP_FACTOR DISTRIBUTION
{
	kahpChannel->caExpFactor = *$2;
	delete $2;
}
| CA_HALF_MIN DISTRIBUTION
{
	kahpChannel->caHalfMin = *$2;
	delete $2;
}
| CA_TAU_SCALE_FACTOR DISTRIBUTION
{
	kahpChannel->caTauScaleFactor = *$2;
	delete $2;
}
| H_INITIAL DISTRIBUTION
{
	kaChannel->hInitial = *$2;
	delete $2;
}
| H_POWER DISTRIBUTION
{
	kaChannel->hPower = *$2;
	delete $2;
}
| E_HALF_MIN_M DISTRIBUTION
{
	if (currentChannel == Km)
		kmChannel->eHalfMinM = *$2;
	else if (currentChannel == Ka)
		kaChannel->eHalfMinM = *$2;
	delete $2;
}
| E_HALF_MIN_H DISTRIBUTION
{
	kaChannel->eHalfMinH = *$2;
	delete $2;
}
| SLOPE_FACTOR_M DISTRIBUTION
{
	kaChannel->slopeFactorM = *$2;
	delete $2;
}
| SLOPE_FACTOR_H DISTRIBUTION
{
	kaChannel->slopeFactorH = *$2;
	delete $2;
}
| V_TAU_VALUE_M NUMBERLIST
{
	kaChannel->vTauValueM = *$2;
	delete $2;
}
| V_TAU_VOLTAGE_M NUMBERLIST
{
	kaChannel->vTauVoltageM = *$2;
	delete $2;
}
| V_TAU_VALUE_H NUMBERLIST
{
	kaChannel->vTauValueH = *$2;
	delete $2;
}
| V_TAU_VOLTAGE_H NUMBERLIST
{
	kaChannel->vTauVoltageH = *$2;
	delete $2;
}

synapse_specification : SYNAPSE { synapse = new Synapse(); }  synapse_parameter_list END_SYNAPSE
{
}

synapse_parameter_list : synapse_parameter synapse_parameter_list
{
}
|
{
}

synapse_parameter : TYPE STRING
{
	synapse->type = *$2;
	delete $2;
}
| SEED INTEGER
{
	synapse->seed = $2;
}
| SFD_LABEL STRING
{
	synapse->sfdLabel = *$2;
	delete $2;
}
| SYN_PSG STRING
{
	synapse->synapticPSG = *$2;
	delete $2;
}
| LEARN_LABEL STRING
{
	synapse->learning = *$2;
	delete $2;
}
| HEBB_START NUMBER
{
	synapse->hebbianOn.push_back($2);
}
| HEBB_END NUMBER
{
	synapse->hebbianOff.push_back($2);
}
| DATA_LABEL STRING
{
//deprecated
}
| ABSOLUTE_USE DISTRIBUTION
{
	synapse->absoluteUSE = *$2;
	delete $2;
}
| RSE_INIT NUMBER NUMBER
{
	synapse->rseInit($2, $3);
}
| PREV_SPIKE_RANGE NUMBER NUMBER
{
	synapse->previousSpikeRange($2, $3);
}
| MAX_CONDUCT DISTRIBUTION
{
	synapse->maxConductance = *$2;
	delete $2;
}
| DELAY NUMBER NUMBER
{
	synapse->delay($2, $3);
}
| SYN_REVERSAL DISTRIBUTION
{
	synapse->synReversal = *$2;
	delete $2;
}

sfd_specification : SYN_FACIL_DEPRESS { rse = new slug::readers::ncs5::file::RSE(); } sfd_parameter_list END_SYN_FACIL_DEPRESS
{
}

sfd_parameter_list : sfd_parameter sfd_parameter_list
{
}
|
{
}

sfd_parameter : TYPE STRING
{
	rse->type = *$2;
	delete $2;
}
| SEED INTEGER
{
	rse->seed = $2;
}
| SFD NONE
{
	rse->depression = false;
	rse->facilitation = false;
}
| SFD DEPR
{
	rse->depression = true;
	rse->facilitation = false;
}
| SFD FACIL
{
	rse->depression = false;
	rse->facilitation = true;
}
| SFD BOTH
{
	rse->depression = true;
	rse->facilitation = true;
}
| FACIL_TAU DISTRIBUTION
{
	rse->tauFacilitation = *$2;
	delete $2;
}
| DEPR_TAU DISTRIBUTION
{
	rse->tauDepression = *$2;
	delete $2;
}

learning_specification : SYN_LEARNING { learning = new Learning(); } learning_parameter_list END_SYN_LEARNING
{
}

learning_parameter_list : learning_parameter learning_parameter_list
{
}
|
{
}

learning_parameter : TYPE STRING 
{
	learning->type = *$2;
	delete $2;
}
| SEED INTEGER
{
	learning->seed = $2;
}
| LEARNING NONE
{
	learning->positive.enable = false;
	learning->negative.enable = false;
}
| LEARNING POS_HEBBIAN
{
	learning->positive.enable = true;
	learning->negative.enable = false;
}
| LEARNING NEG_HEBBIAN
{
	learning->positive.enable = false;
	learning->negative.enable = true;
}
| LEARNING BOTH
{
	learning->positive.enable = true;
	learning->negative.enable = true;
}
| LEARNING_SHAPE TRIANGLE
{
	learning->shape = Learning::Triangle;
}
| LEARNING_SHAPE EXPONENT
{
	learning->shape = Learning::Exponent;
}
| NEG_HEB_WINDOW DISTRIBUTION
{
	learning->negative.window = *$2;
	delete $2;
}
| POS_HEB_WINDOW DISTRIBUTION
{
	learning->positive.window = *$2;
	delete $2;
}
| POS_HEB_PEAK_DELTA_USE DISTRIBUTION
{
	learning->positive.deltaUSE = *$2;
	delete $2;
}
| NEG_HEB_PEAK_DELTA_USE DISTRIBUTION
{
	learning->negative.deltaUSE = *$2;
	delete $2;
}
| POS_HEB_PEAK_TIME DISTRIBUTION
{
	learning->positive.peakTime = *$2;
	delete $2;
}
| NEG_HEB_PEAK_TIME DISTRIBUTION
{
	learning->negative.peakTime = *$2;
	delete $2;
}

psg_specification : SYN_PSG { psg = new PSG(); }  psg_parameter_list END_SYN_PSG
{
}

psg_parameter_list : psg_parameter psg_parameter_list
{
}
|
{
}

psg_parameter : TYPE STRING
{
	psg->type = *$2;
	delete $2;
}
| PSG_FILE STRING
{
	std::string file = *$2;
	if (file[0] != '/')
	{
		file = currentPath + "/" + file;
	}
	psg->filename = file;
	delete $2;
}

augmentation_specification : SYN_AUGMENTATION {augmentation = new SynapticAugmentation(); }  augmentation_parameter_list END_SYN_AUGMENTATION
{
}

augmentation_parameter_list : augmentation_parameter augmentation_parameter_list
{
}
|
{
}

augmentation_parameter : TYPE STRING
{
	augmentation->type = *$2;
	delete $2;
}
| AUGMENTATION_INIT DISTRIBUTION
{
	augmentation->initAugmentation = *$2;
	delete $2;
}
| AUGMENTATION_TAU DISTRIBUTION
{
	augmentation->tauAugmentation = *$2;
	delete $2;
}
| MAX_AUGMENTATION DISTRIBUTION
{
	augmentation->maxAugmentation = *$2;
	delete $2;
}
| CA_INTERNAL DISTRIBUTION
{
	augmentation->caInternal = *$2;
	delete $2;
}
| CA_SPIKE_INCREMENT DISTRIBUTION
{
	augmentation->caSpikeIncrement = *$2;
	delete $2;
}
| CA_TAU DISTRIBUTION
{
	augmentation->caTau = *$2;
	delete $2;
}
| ALPHA DISTRIBUTION
{
	augmentation->alpha = *$2;
	delete $2;
}

spikeshape_specification : SPIKESHAPE { spikeShape = new SpikeShape(); }  spikeshape_parameter_list END_SPIKESHAPE
{
}

spikeshape_parameter_list : spikeshape_parameter spikeshape_parameter_list
{
}
|
{
}

spikeshape_parameter : TYPE STRING
{
	spikeShape->type = *$2;
	delete $2;
}
| VOLTAGES NUMBERLIST
{
	spikeShape->voltages = *$2;
	delete $2;
}

stimulus_specification : STIMULUS { stimulus = new Stimulus(); }  stimulus_parameter_list END_STIMULUS
{
}

stimulus_parameter_list : stimulus_parameter stimulus_parameter_list
{
}
|
{
}

stimulus_parameter : TYPE STRING
{
	stimulus->type = *$2;
	delete $2;
}
| MODE STRING
{
	if (*$2 == "VOLTAGE")
	{
		stimulus->mode = Stimulus::Voltage;
	}
	else if (*$2 == "CURRENT")
	{
		stimulus->mode = Stimulus::Current;
	}
	else
	{
		yyerror("Mode unrecognized");
		exit(-1);
	}
	delete $2;
}
| PATTERN STRING
{
	if (*$2 == "LINEAR")
	{
		stimulus->pattern = Stimulus::Linear;
	}
	else if (*$2 == "PULSE")
	{
		stimulus->pattern = Stimulus::Pulse;
	}
	else if (*$2 == "FILE_BASED")
	{
		stimulus->pattern = Stimulus::FileBased;
	}
	else if (*$2 == "INPUT")
	{
		stimulus->pattern = Stimulus::SocketBased;
	}
	else
	{
		yyerror("Unrecognized pattern");
		exit(-1);
	}
	delete $2;
}
| FILENAME STRING
{
	std::string file = *$2;
	if (file[0] != '/')
	{
		file = currentPath + "/" + file;
	}
	stimulus->filename = file;
	stimulus->originalName = *$2;
	delete $2;
}
| PORT INTEGER
{
	stimulus->port = $2;
}
| PORT AUTO
{
//TODO
}
| TIME_INCREMENT NUMBER
{
	stimulus->timeIncrement = $2;
}
| FREQ_ROWS INTEGER
{
	yyerror("This is deprecated. Get rid of it");
	exit(-1);
}
| FREQ_COLS INTEGER
{
	stimulus->frequencyColumns = $2;
}
| CELLS_PER_FREQ INTEGER
{
	stimulus->cellsPerFrequency = $2;
}
| DYN_RANGE NUMBER NUMBER
{
	stimulus->dynamicRange($2, $3);
}
| TIMING STRING
{
	if (*$2 == "EXACT")
		stimulus->timing = Stimulus::Timing::Exact;
	else if (*$2 == "URAND")
		stimulus->timing = Stimulus::Timing::URand;
	else if (*$2 == "POISSON")
		stimulus->timing = Stimulus::Timing::Poisson;
	else
	{
		yyerror("Unrecognized timing");
		exit(-1);
	}
}
| SAMESEED YES
{
	stimulus->sameseed = true;
}
| SAMESEED NO
{
	stimulus->sameseed = false;
}
| AMP_START NUMBER
{
	stimulus->startingAmplitude = $2;
}
| AMP_END NUMBER
{
	stimulus->endingAmplitude = $2;
}
| PHASE NUMBER
{
	stimulus->phase = $2;
}
| VERT_TRANS NUMBER
{
	stimulus->verticalTranslation = $2;
}
| WIDTH NUMBER
{
	stimulus->width = $2;
}
| TIME_START NUMBERLIST
{
	stimulus->startTimes = *$2;
	delete $2;
}
| TIME_END NUMBERLIST
{
	stimulus->endTimes = *$2;
	delete $2;
}
| FREQ_START NUMBER
{
	stimulus->startingFrequency = $2;
}
| SEED INTEGER
{
	stimulus->seed = $2;
}
| RATE NUMBER
{
	stimulus->rate = $2;
}
| TAU_NOISE NUMBER
{
	stimulus->tauNoise = $2;
}
| CORREL NUMBER
{
	stimulus->correlation = $2;
}

stimulus_inject_specification : STIMULUS_INJECT { injection = new StimulusInject(); } stimulus_inject_parameter_list END_STIMULUS_INJECT
{
}

stimulus_inject_parameter_list : stimulus_inject_parameter stimulus_inject_parameter_list
{
}
|
{
}

stimulus_inject_parameter : TYPE STRING
{
	injection->type = *$2;
	delete $2;
}
| STIM_TYPE STRING
{
	injection->stimulusType = *$2;
	delete $2;
}
| INJECT STRING STRING STRING STRING NUMBER
{
	injection->injection.column = *$2;
	injection->injection.layer = *$3;
	injection->injection.cellType = *$4;
	injection->injection.compartment = *$5;
	injection->injection.probability = $6;
	delete $2;
	delete $3;
	delete $4;
	delete $5;
}

report_specification : REPORT {report = new Report(); } report_parameter_list END_REPORT
{
}

report_parameter_list : report_parameter report_parameter_list
{
}
|
{
}

report_parameter : TYPE STRING
{
	report->type = *$2;
	delete $2;
}
| CELLS STRING STRING STRING STRING
{
	Report::Source& s = report->source;
	s.column = *$2;
	s.layer = *$3;
	s.cellType = *$4;
	s.compartment = *$5;
	delete $2;
	delete $3;
	delete $4;
	delete $5;
}
| PROB NUMBER
{
	report->probability = $2;	
}
| CELL_SEQUENCE STRING
{
	report->cellSequence = (*$2 == "RANDOM");
	delete $2;
}
| FREQUENCY INTEGER
{
	report->frequency = $2;
}
| REPORT_ON STRING
{
	std::string s = *$2;
	delete $2;
	if (s == "VOLTAGE")
		report->reportType = 
			slug::readers::ncs5::file::Report::ReportType::Voltage;
	else if (s == "FIRE_COUNT")
		report->reportType = 
			slug::readers::ncs5::file::Report::ReportType::FireCount;
	else if (s == "SYN_CURRENT")
		report->reportType =
			slug::readers::ncs5::file::Report::ReportType::SynapticCurrent;
	else if (s == "STIM_CURRENT")
		report->reportType =
			slug::readers::ncs5::file::Report::ReportType::StimulusCurrent;
	else
		report->reportType = 
			slug::readers::ncs5::file::Report::ReportType::Other;
	//TODO: Parse the crazy
}
| CHANNEL STRING 
{
}
| SYNAPSE STRING
{
	report->reportType =
		slug::readers::ncs5::file::Report::ReportType::USE;
	report->synapseUSE = *$2;
	delete $2;
}
| SYNAPSE_RSE STRING
{
}
| SYNAPSE_UF STRING
{
}
| SYN_AUGMENTATION STRING
{
}
| ASCII
{
	report->ascii = true;
	report->exponential = false;
}
| ASCII EXP
{
	report->ascii = true;
	report->exponential = true;
}
| FILENAME STRING
{
	report->filename = *$2;
	delete $2;
}
| TIME_START NUMBERLIST
{
	report->startingTimes = *$2;
	delete $2;
}
| TIME_END NUMBERLIST
{
	report->endingTimes = *$2;
	delete $2;
}
| PORT AUTO
{
	report->networked = true;
}


event_specification : EVENT { event = new Event(); } event_parameter_list END_EVENT
{
}

event_parameter_list : event_parameter event_parameter_list
{
}
|
{
}

event_parameter : TYPE STRING
{
	event->type = *$2;
	delete $2;
}
| USE_OVERRIDE STRING NUMBER
{
	event->filename = *$2;
	event->time = $3;
	delete $2;
}
| CELLS STRING STRING STRING STRING
{
	event->column = *$2;
	event->layer = *$3;
	event->cellType = *$4;
	event->compartment = *$5;
	delete $2;
	delete $3;
	delete $4;
	delete $5;
}
| SYNAPSE STRING
{
	event->synapse = *$2;
	delete $2;
}

NUMBER : FLOAT
{
	$$ = $1; 
}
| INTEGER
{
	$$ = $1;
}

NUMBERLIST : NUMBERLIST NUMBER
{
	$1->push_back($2);
	$$ = $1;
}
| NUMBER
{
	$$ = new std::vector<float>();
	$$->push_back($1);
}

DISTRIBUTION : NUMBER NUMBER
{
	$$ = new NormalDistribution($1, $2);
}
| NUMBER
{
	$$ = new NormalDistribution($1, 0.0f);
}
