%{

#define YY_INPUT(buf, result, max_size)\
{\
	int c = getc(yyin); \
	result = (c == EOF) ? YY_NULL : (buf[0] = c, 1);\
}

#include <vector>
#include <stack>
#include <string>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <parser.hpp>
#include <iostream>

long lineNumber = 1;
bool outputScannerDebug = true;

void debugs(const char* message, ...)
{
	if (outputScannerDebug)
	{
		va_list args;
		va_start(args, message);
		vfprintf(stdout, message, args);
		va_end(args);
	}
}

%}

%option nounput

float          (-?[0-9]*\.[0-9]+|-?[0-9]+\.[0-9]*)(e-?[0-9]*\.[0-9]+|-?[0-9]+\.[0-9]*)?
string         [./A-Za-z_][\[\]A-Za-z0-9/\-\._:]*
quoted_string  \"[\[\]A-Za-z0-9/\-._: =\$;~]*\"
integer		   [0-9]+|-[0-9]+
comment        #[^\n]*$
linebreaks     \r\n|\n
whitespace     [\t ]+

%%

\+HEBBIAN { return POS_HEBBIAN; }
\-HEBBIAN { return NEG_HEBBIAN; }
ABSOLUTE_USE { return ABSOLUTE_USE; }           
ALPHA { return ALPHA; }                  
ALPHA_SCALE_FACTOR_H { return ALPHA_SCALE_FACTOR_H; }   
ALPHA_SCALE_FACTOR_M { return ALPHA_SCALE_FACTOR_M; }   
AMP_END { return AMP_END; }                
AMP_START { return AMP_START; }              
ASCII { return ASCII; }                  
AUTO { return AUTO; }                   
AUGMENTATION_DELAY { return AUGMENTATION_DELAY; }     
AUGMENTATION_INIT { return AUGMENTATION_INIT; }      
AUGMENTATION_TAU { return AUGMENTATION_TAU; }       
AVERAGE_SYN { return AVERAGE_SYN; }            
BETA_SCALE_FACTOR_H { return BETA_SCALE_FACTOR_H; }    
BETA_SCALE_FACTOR_M { return BETA_SCALE_FACTOR_M; }    
BOTH { return BOTH; }
BRAIN { return BRAIN; }                  
CA_EXP_FACTOR { return CA_EXP_FACTOR; }          
CA_EXP_GAUSS { return CA_EXP_GAUSS; }           
CA_EXP_RANGE { return CA_EXP_RANGE; }           
CA_EXTERNAL { return CA_EXTERNAL; }            
CA_HALF_MIN { return CA_HALF_MIN; }            
CA_INTERNAL { return CA_INTERNAL; }            
CA_SCALE_FACTOR { return CA_SCALE_FACTOR; }        
CA_SPIKE_INCREMENT { return CA_SPIKE_INCREMENT; }     
CA_TAU { return CA_TAU; }                 
CA_TAU_DECAY { return CA_TAU_DECAY; }           
CA_TAU_SCALE_FACTOR { return CA_TAU_SCALE_FACTOR; }    
CELL { return CELL; }                   
CELL_SEQUENCE { return CELL_SEQUENCE; }          
CELLS { return CELLS; }                  
CELL_TYPE { return CELL_TYPE; }              
CHANNEL { return CHANNEL; }                
COLUMN { return COLUMN; }                 
COLUMN_SHELL { return COLUMN_SHELL; }           
COLUMN_TYPE { return COLUMN_TYPE; }            
COMPARTMENT { return COMPARTMENT; }            
MAX_CONDUCT { return MAX_CONDUCT; }            
CONNECT { return CONNECT; }                
DATA_LABEL { return DATA_LABEL; }             
DELAY { return DELAY; }                  
DEPR { return DEPR; }
DEPR_TAU { return DEPR_TAU; }               
DISTANCE { return DISTANCE; }               
DURATION { return DURATION; }               
END_BRAIN { return END_BRAIN; }              
END_CELL { return END_CELL; }               
END_CHANNEL { return END_CHANNEL; }            
END_COLUMN { return END_COLUMN; }             
END_COLUMN_SHELL { return END_COLUMN_SHELL; }       
END_COMPARTMENT { return END_COMPARTMENT; }        
END_EVENT { return END_EVENT; }              
END_LAYER { return END_LAYER; }              
END_LAYER_SHELL { return END_LAYER_SHELL; }        
END_REPORT { return END_REPORT; }             
END_SPIKESHAPE { return END_SPIKESHAPE; }         
END_STIMULUS { return END_STIMULUS; }           
END_STIMULUS_INJECT { return END_STIMULUS_INJECT; }    
END_SYNAPSE { return END_SYNAPSE; }            
END_SYN_AUGMENTATION { return END_SYN_AUGMENTATION; }   
END_SYN_DATA { return END_SYN_DATA; }           
END_SYN_FACIL_DEPRESS { return END_SYN_FACIL_DEPRESS; }  
END_SYN_LEARNING { return END_SYN_LEARNING; }       
END_SYN_PSG { return END_SYN_PSG; }            
EXP { return EXP; }                    
EXPONENT { return EXPONENT; }
EVENT { return EVENT; }                  
E_HALF_MIN_H { return E_HALF_MIN_H; }           
E_HALF_MIN_M { return E_HALF_MIN_M; }           
FACIL { return FACIL; }
FACIL_TAU { return FACIL_TAU; }              
FILENAME { return FILENAME; }               
FREQUENCY { return FREQUENCY; }              
FREQ_ROWS { return FREQ_ROWS; }              
FREQ_COLS { return FREQ_COLS; }              
FREQ_START { return FREQ_START; }             
FSV { return FSV; }                    
HEBB_ON { return HEBB_ON; }                
HEBB_OFF { return HEBB_OFF; }               
HEBB_START { return HEBB_START; }             
HEBB_END { return HEBB_END; }               
HEIGHT { return HEIGHT; }                 
HIDE_TIMESTEP { return HIDE_TIMESTEP; }          
H_INITIAL { return H_INITIAL; }              
H_POWER { return H_POWER; }                
IGNORE_EMPTY { return IGNORE_EMPTY; }           
INCLUDE { return INCLUDE; }                
INJECT { return INJECT; }                 
INTERACTIVE { return INTERACTIVE; }            
LAYER { return LAYER; }                  
LAYER_SHELL { return LAYER_SHELL; }            
LAYER_TYPE { return LAYER_TYPE; }             
LEAK_CONDUCTANCE { return LEAK_CONDUCTANCE; }       
LEAK_REVERSAL { return LEAK_REVERSAL; }          
LEARNING { return LEARNING; }               
LEARNING_SHAPE { return LEARNING_SHAPE; }         
LEARN_LABEL { return LEARN_LABEL; }            
LOAD { return LOAD; }                   
LOCATION { return LOCATION; }               
LOWER { return LOWER; }                  
MAX_AUGMENTATION { return MAX_AUGMENTATION; }       
MAX_G { return MAX_G; }                  
MODE { return MODE; }                   
M_INITIAL { return M_INITIAL; }              
M_POWER { return M_POWER; }                
NEG_HEB_PEAK_DELTA_USE { return NEG_HEB_PEAK_DELTA_USE; } 
NEG_HEB_PEAK_TIME { return NEG_HEB_PEAK_TIME; }      
NEG_HEB_WINDOW { return NEG_HEB_WINDOW; }         
NONE { return NONE; }
OUTPUT_CELLS { return OUTPUT_CELLS; }           
OUTPUT_CONNECT_MAP { return OUTPUT_CONNECT_MAP; }     
OPTION { return OPTION; }                 
PATTERN { return PATTERN; }                
PHASE { return PHASE; }            
PHASE_SHIFT { return PHASE_SHIFT; }            
POS_HEB_PEAK_DELTA_USE { return POS_HEB_PEAK_DELTA_USE; } 
POS_HEB_PEAK_TIME { return POS_HEB_PEAK_TIME; }      
POS_HEB_WINDOW { return POS_HEB_WINDOW; }         
PREV_SPIKE_RANGE { return PREV_SPIKE_RANGE; }       
PROB { return PROB; }                   
RECURRENT_CONNECT { return RECURRENT_CONNECT; }      
RELOAD_SYN { return RELOAD_SYN; }             
REPORT { return REPORT; }                 
REPORT_ON { return REPORT_ON; }              
REVERSAL_POTENTIAL { return REVERSAL_POTENTIAL; }     
RSE { return RSE; }                    
RSE_LABEL { return RSE_LABEL; }              
RSE_INIT { return RSE_INIT; }               
R_MEMBRANE { return R_MEMBRANE; }             
SAMESEED { return SAMESEED; }               
SAVE { return SAVE; }                   
SAVE_SYN { return SAVE_SYN; }               
SEED { return SEED; }                   
SINGLE { return SINGLE; }                 
SELECT_FRONT { return SELECT_FRONT; }           
SERVER { return SERVER; }                 
SERVER_PORT { return SERVER_PORT; }            
SLOPE_FACTOR_H { return SLOPE_FACTOR_H; }         
SLOPE_FACTOR_H_STDEV { return SLOPE_FACTOR_H_STDEV; }   
SLOPE_FACTOR_M { return SLOPE_FACTOR_M; }         
SLOPE_FACTOR_M_STDEV { return SLOPE_FACTOR_M_STDEV; }   
SPIKESHAPE { return SPIKESHAPE; }             
SPIKE_HALFWIDTH { return SPIKE_HALFWIDTH; }        
STIMULUS { return STIMULUS; }               
STIMULUS_INJECT { return STIMULUS_INJECT; }        
STIM_TYPE { return STIM_TYPE; }              
STRENGTH { return STRENGTH; }               
STRENGTH_GAUSS { return STRENGTH_GAUSS; }         
STRENGTH_RANGE { return STRENGTH_RANGE; }         
SYNAPSE { return SYNAPSE; }                
SYNAPSE_RSE { return SYNAPSE_RSE; }            
SYNAPSE_UF { return SYNAPSE_UF; }             
SYN_AUGMENTATION { return SYN_AUGMENTATION; }       
SYN_CALCIUM { return SYN_CALCIUM; }            
SYN_DATA { return SYN_DATA; }               
SYN_FACIL_DEPRESS { return SYN_FACIL_DEPRESS; }      
SYN_LEARNING { return SYN_LEARNING; }           
SYN_PSG { return SYN_PSG; }                
SYN_REVERSAL { return SYN_REVERSAL; }           
TAU_MEMBRANE { return TAU_MEMBRANE; }           
TAU_SCALE_FACTOR_H { return TAU_SCALE_FACTOR_H; }     
TAU_SCALE_FACTOR_M { return TAU_SCALE_FACTOR_M; }     
THRESHOLD { return THRESHOLD; }              
TIME_END { return TIME_END; }               
TIME_START { return TIME_START; }             
TIMING { return TIMING; }                 
TRIANGLE { return TRIANGLE; }
TYPE { return TYPE; }                   
UNITARY_G { return UNITARY_G; }              
UPPER { return UPPER; }                  
USE_OVERRIDE { return USE_OVERRIDE; }           
VERT_TRANS { return VERT_TRANS; }             
VMREST { return VMREST; }                 
VOLTAGES { return VOLTAGES; }               
V_TAU_VALUE_H { return V_TAU_VALUE_H; }          
V_TAU_VOLTAGE_H { return V_TAU_VOLTAGE_H; }        
V_TAU_VALUE_M { return V_TAU_VALUE_M; }          
V_TAU_VOLTAGE_M { return V_TAU_VOLTAGE_M; }        
V_TAU_VALUE_H_STDEV { return V_TAU_VALUE_H_STDEV; }    
V_TAU_VOLTAGE_H_STDEV { return V_TAU_VOLTAGE_H_STDEV; }  
V_TAU_VALUE_M_STDEV { return V_TAU_VALUE_M_STDEV; }    
V_TAU_VOLTAGE_M_STDEV { return V_TAU_VOLTAGE_M_STDEV; }  
VERSION { return VERSION; }                
WARNINGS_OFF { return WARNINGS_OFF; }           
WIDTH { return WIDTH; }                  
SFD { return SFD; }                    
SFD_LABEL { return SFD_LABEL; }              
PSG_FILE { return PSG_FILE; }               
CELLS_PER_FREQ { return CELLS_PER_FREQ; }         
TIME_INCREMENT { return TIME_INCREMENT; }         
DYN_RANGE { return DYN_RANGE; }              
PORT { return PORT; }                   
JOB { return JOB; }                    
DISTRIBUTE { return DISTRIBUTE; }             
CONNECT_REPORT { return CONNECT_REPORT; }         
SPIKE_REPORT { return SPIKE_REPORT; }           
RATE { return RATE; }                   
TAU_NOISE { return TAU_NOISE; }              
CORREL { return CORREL; }                 
NO { return NO; }
YES { return YES; }

{float} {
	yylval.floatingPoint = atof(yytext);
	return FLOAT;
}

{string} {
	yylval.string = new std::string(yytext);
	return STRING;
}

{integer} {
	yylval.integer = atoi(yytext);
	return INTEGER;
}

{whitespace} {
}

{comment} {
}

{linebreaks} {
	++lineNumber;
}

. {
	return static_cast<int>(*yytext);
}

%%

extern std::stack<std::string> filenameStack;
extern std::stack<long> lineNumberStack;
extern std::stack<FILE*> fileStack;
extern std::stack<std::string> pathStack;

extern FILE* yyin;
std::string currentFilename;
std::string currentPath;

int yywrap(void)
{
	if (!fileStack.empty())
	{
		fclose(yyin);

		yyin = fileStack.top();
		lineNumber = lineNumberStack.top();
		currentFilename = filenameStack.top();
		currentPath = pathStack.top();

		filenameStack.pop();
		fileStack.pop();
		lineNumberStack.pop();
		pathStack.pop();

		return 0;
	}

	return 1;
}

