/*
	Implementation file for InputUpdater.h
	namespace slug::sim
*/

template<util::MemoryType MType>
InputUpdater<MType>::InputUpdater(unsigned int voltageVectorSize,
             std::vector<InputSimulator<MType>*> simulators,
             unsigned int numBuffers)
	: _voltageVectorSize(voltageVectorSize),
	  _simulators(simulators),
	  _numBuffers(numBuffers)
{
	//Allocate blanks
	for (unsigned int i = 0; i < _numBuffers; ++i)
	{
		auto buffer = new InputBuffer<MType>(_voltageVectorSize);
		_addBlank(buffer);
	}
}

template<util::MemoryType MType>
bool InputUpdater<MType>::init(const SimulationProperties* properties)
{
	//Make sure simulators are ready to go
	for (auto sim : _simulators)
	{
		if (!sim->init(properties))
			return false;
	}
	return true;
}

template<util::MemoryType MType>
bool InputUpdater<MType>::run(const SimulationProperties* properties)
{
	Device* device = Device::device;
	auto threadInitializer = [=](InputSimulator<MType>* sim)
	{
		device->threadInit();
	};

	auto updateFunction = [](InputSimulator<MType>* simulator,
	                         InputBuffer<MType>* buffer)
	{
		simulator->update(buffer);
	};

	auto publishFunction = [this](InputBuffer<MType>* buffer)
	{
		this->publish(buffer);
	};

	auto threadDestructor = [device](InputSimulator<MType>*)
	{
		device->threadDestroy();
	};

	bool result = Updater<InputBuffer<MType>>::template 
		run<InputSimulator<MType>>(_simulators,
	                               threadInitializer,
				                   updateFunction,
								   publishFunction,
								   threadDestructor,
				                   _numBuffers,
								   properties);
	return result;
}

