/**
	Template implementation file for VectorExchanger.h
	namespace slug::sim
*/

template<util::MemoryType MType>
SpecificDeviceVectorExchanger<MType>::
SpecificDeviceVectorExchanger(unsigned int bitVectorSize,
                              unsigned int numBuffers)
	: _numBuffers(numBuffers)
{
	//Allocate blanks 
	for (unsigned int i = 0; i < numBuffers; ++i)
	{
		auto buffer = new NeuronFireVectorBuffer<MType>(bitVectorSize);
		_addBlank(buffer);
	}
}

template<util::MemoryType MType>
void SpecificDeviceVectorExchanger<MType>::pull(unsigned int* dst)
{
	auto* buffer = _stateSubscription->pull();
	this->template _transfer<util::MemoryType::System, MType, unsigned int>
		(dst, buffer->neuronFireVector, buffer->bitVectorSize);
	buffer->release();
}

template<util::MemoryType MType>
bool SpecificDeviceVectorExchanger<MType>::
run(const SimulationProperties* properties)
{
	unsigned int numSteps = properties->numSteps;
	for (unsigned int i = 0; i < numSteps; ++i)
	{
		//Get the partial vector
		auto input = _vectorSubscription->pull();

		//Move it to system memory
		auto output = this->_getBlank();
		this->template 
			_transfer<MType, util::MemoryType::System, unsigned int>
				(output->neuronFireVector, 
				 input->neuronFireVector,
				 input->bitVectorSize);
		publish(output);
		input->release();
	}
	return true;
}

template<util::MemoryType MType>
void SpecificDeviceVectorExchanger<MType>::
statePublisher(SpecificPublisher<NeuronStateBuffer<MType>>* pub)
{
	_stateSubscription = pub->subscribe();
}

template<util::MemoryType MType>
SpecificDeviceVectorExchanger<MType>::~SpecificDeviceVectorExchanger()
{
	if (_stateSubscription)
	{
		delete _stateSubscription;
		_stateSubscription = nullptr;
	}
}

template<util::MemoryType MType>
void SpecificDeviceVectorExchanger<MType>::
vectorPublisher(NeuronFireVectorBufferPublisher* pub)
{
	_vectorSubscription = pub->subscribe();
}
