template<typename T>
Parameter<T>::Parameter(const T& value)
	: _value(value)
{
}

template<typename T>
BaseParameter* Parameter<T>::copy() const
{
	return new Parameter(_value);
}

template<typename T>
size_t Parameter<T>::size() const
{
	return sizeof(T);
}

template<typename T>
bool Parameter<T>::write(std::ostream& os) const
{
	os.write((char*)&_value, sizeof(T));
	return os.good();
}

template<typename T> 
bool Specification::set(util::Unique id, const T& v)
{
	if (_parameters.find(id) != _parameters.end())
	{
		delete _parameters[id];
	}
	_parameters[id] = new Parameter<T>(v);
	return true;
}

template<typename T> 
bool Specification::set(util::Unique id, T* v, size_t size)
{
	if (_parameters.find(id) != _parameters.end())
	{
		delete _parameters[id];
	}
	_parameters[id] = new ArrayParameter(v, size * sizeof(T));
	return true;
}

template<typename T> 
bool Specification::set(const std::string& name, const T& v)
{
	util::Unique id = util::StringID::get(name);
	return set(id, v);
}

template<typename T> 
bool Specification::set(const std::string& name, T* v, size_t size)
{
	util::Unique id = util::StringID::get(name);
	return set(id, v, size);
}
