#include <dirent.h>
#include <dlfcn.h>
#include <iostream>
#include <slug/sim/Plugin.h>

using namespace slug::sim;

/**
	Loads a directory of plugins.
*/
bool Plugin::loadDirectory(const std::string& path)
{
	//Open the directory
	DIR* directory = opendir(path.c_str());
	if (nullptr == directory)
	{
		std::cerr << "Could not open directory " << path << std::endl;
		return false;
	}

	//Go through every file fand try to load it
	for (struct dirent* file = readdir(directory);
	     file != nullptr; file = readdir(directory))
	{
		if (file->d_type == DT_REG)
			loadPlugin(path + "/" + file->d_name);
	}
	closedir(directory);
	return true;
}

/**
	Loads a single plugin.
*/
bool Plugin::loadPlugin(const std::string& path)
{
	//Open the file
	void* handle = dlopen(path.c_str(), RTLD_NOW | RTLD_LOCAL);
	if (nullptr == handle)
	{
		std::cerr << "Could not open file " << path << ": " <<
		    dlerror() << std::endl;
		return false;
	}

	//Get the activate function
	void* (*function)();
	*(void**)(&function) = dlsym(handle, "activate");
	if (nullptr == function)
	{
		dlclose(handle);
		return false;
	}

	//Invoke the function
	(*function)();
	return true;
}

Plugin::Plugin(const std::string& __type,
       const std::string& __name,
	   unsigned int __revision)
	: _type(__type),
	  _name(__name),
	  _revision(__revision)
{
}

/**
	Returns the service this plugin offers.
*/
const std::string& Plugin::type()
{
	return _type;
}

/**
	Returns the full name of this plugin. Can be used to differentiate
	between similar plugins.
*/
const std::string& Plugin::name() const
{
	return _name;
}

/**
	Returns a revision number. Higher revision numbers are preferred over
	older revision numbers.
*/
unsigned int Plugin::revision() const
{
	return _revision;
}

const std::set<std::string>& Plugin::features() const
{
	return _features;
}

/**
	Utility function for derived classes to add features to the feature
	set. Should be called during construction.
*/
void Plugin::addFeature(const std::string& f)
{
	_features.insert(f);
}

