#include <slug/sim/Reporter.h>

using namespace slug;
using namespace slug::sim;

//Manager instantiation
template<> Reporters* Reporters::_instance = nullptr;

Reporter* Reporter::read(const std::string& path)
{
	//Open the file
	util::InputStream bs(path);
	if (bs.fail())
		return nullptr;
	
	//Get the type of report
	std::string type;
	bs >> type;

	//Try to find a plugin that can handle this type of report
	auto factory = Reporters::get(type);
	if (nullptr == factory)
	{
		std::cerr << "No plugin found for report of type " << type <<
			". Aborting report loading." << std::endl;
		return nullptr;
	}

	//Generate an actual reporter
	auto report = factory->generate();

	//Get the number of machines involved
	unsigned int numMachines = 0;
	bs >> numMachines;

	//Read the generic parameter interface
	report->Parameterized::read(bs);

	//Get the ID of the master machine for this group
	bs >> report->masterID;

	//For each machine
	for (unsigned int i = 0; i < numMachines; ++i)
	{
		//Generate a spec
		Reporter::MachineSpec* machineSpec = new Reporter::MachineSpec;

		//Get the machine ID
		bs >> machineSpec->id;

		//Get the number of reportables
		unsigned int numReportables = 0;
		bs >> numReportables;

		//For each reportable
		for (unsigned int j = 0; j < numReportables; ++j)
		{
			//Generate a reportable spec
			Reporter::MachineSpec::ReportableSpec* reportableSpec = 
				new Reporter::MachineSpec::ReportableSpec;

			//Get the reportable ID
			bs >> reportableSpec->id;

			//Get the number of indices on the reportable
			unsigned int numIndices = 0;
			bs >> numIndices;

			//Read in all the indices
			for (unsigned int k = 0; k < numIndices; ++k)
			{
				unsigned int index = 0;
				bs >> index;
				reportableSpec->indices.push_back(index);
			}

			//Read in the user ID mappings
			for (unsigned int k = 0; k < numIndices; ++k)
			{
				unsigned int userID = 0;
				bs >> userID;
				reportableSpec->userIDs.push_back(userID);
			}

			//Add the reportable spec
			machineSpec->reportables.push_back(reportableSpec);
		}

		//Add the machine spec
		report->machines.push_back(machineSpec);
	}
	bs.close();
	
	return report;
}

bool Reporter::involves(util::Unique machineID) const
{
	for (auto spec : machines)
		if (spec->id == machineID)
			return true;
	return false;
}

bool Reporter::machineInit(util::Unique machineID)
{
	for (auto spec : machines)
	{
		if (spec->id == machineID)
		{
			master = (machineID == masterID);
			mySpec = spec;
			return true;
		}
	}
	std::cerr << "Could not find machine ID " << machineID <<
		std::endl;
	return false;
}

bool Reporter::initCommunications(BrainSimulator* bs)
{
	std::vector<util::Unique> ids;
	for (auto spec : machines)
		ids.push_back(spec->id);

	communicator = bs->createCommunicator(ids);
	return true;
}
