#include <slug/sim/cuda/FireTableUpdate.h>
#include <slug/sim/CUDA.h>
#include <slug/util/CUDA.h>
#include <algorithm>
#include <stdio.h>

__global__  
void updateFireTableKernel(unsigned int timeStep,
                           unsigned int* neuronFireVector,
						   unsigned int neuronFireVectorSize,
					       unsigned int* synapseFireTable,
					       unsigned int synapticVectorSize,
					       unsigned int row,
					       unsigned int numRows,
					       unsigned int* presynapticNeuronIDs,
					       unsigned int* synapticDelays,
					       unsigned int numSynapses)
{
#if 0
	extern __shared__ unsigned int neuronVector[];
	//Load the vector into shmem
	{
		unsigned int index = block::thread();
		unsigned int stride = block::stride();
		for (; index < neuronFireVectorSize; index += stride)
		{
			neuronVector[index] = neuronFireVector[index];
		}
	}
	__syncthreads();
#endif
	//Update the table
	unsigned int index = grid::thread();
	unsigned int stride = grid::stride();
	for (; index < numSynapses; index += stride)
	{
		unsigned int preID = presynapticNeuronIDs[index];
		if (preID == 0xFFFFFFFF)
			continue;
		unsigned int word = bit::word(preID);
		unsigned int mask = bit::mask(preID);
		//if (neuronVector[word] & mask) //pre neuron fired
		if (neuronFireVector[word] & mask) //pre neuron fired
		{
			unsigned int delay = synapticDelays[index];
			unsigned int eventRow = row + delay;
			if (eventRow >= numRows)
				eventRow -= numRows;
			unsigned int eventMask = bit::mask(index);
			unsigned int* eventWord =
			    synapseFireTable + eventRow * synapticVectorSize;
			atomicOr(eventWord, eventMask);
		}
	}
}

void updateFireTable(unsigned int timeStep,
                     unsigned int* neuronFireVector,
					 unsigned int neuronFireVectorSize,
					 unsigned int* synapseFireTable,
					 unsigned int synapticVectorSize,
					 unsigned int row,
					 unsigned int numRows,
					 unsigned int* presynapticNeuronIDs,
					 unsigned int* synapticDelays,
					 unsigned int numSynapses)
{
	unsigned int threadsPerBlock = 512;
	unsigned int blocksPerGrid =
	    (numSynapses + threadsPerBlock - 1) / threadsPerBlock;
	blocksPerGrid = std::min(blocksPerGrid, 256u);
	unsigned int shmemSize = 0;
	//shmemSize += neuronFireVectorSize * sizeof(unsigned int);
	updateFireTableKernel<<<blocksPerGrid, 
	                        threadsPerBlock,
							shmemSize,
							slug::sim::CUDA::stream>>>
	    (timeStep,
		 neuronFireVector,
		 neuronFireVectorSize,
		 synapseFireTable,
		 synapticVectorSize,
		 row,
		 numRows,
		 presynapticNeuronIDs,
		 synapticDelays,
		 numSynapses);
	slug::sim::CUDA::synchronize();
}
