#include <slug/sim/DataBuffer.h>
#include <slug/sim/Device.h>

using namespace slug::sim;

Publisher::Publisher()
{
	device = Device::device;
}

Publisher::Subscription* Publisher::subscribe()
{
	Subscription* sub = new Subscription(this);
	std::unique_lock<std::mutex> lock(_mutex);
	_subscriptions.push_back(sub);
	return sub;
}

void Publisher::unsubscribe(Publisher::Subscription* sub)
{
	std::unique_lock<std::mutex> lock(_mutex);
	for (auto it = _subscriptions.begin(); it != _subscriptions.end(); ++it)
	{
		if (*it == sub)
		{
			_subscriptions.erase(it);
			break;
		}
	}
}

unsigned int Publisher::publish(DataBuffer* db)
{
	std::unique_lock<std::mutex> lock(_mutex);
	db->subscriptionCount += _subscriptions.size();
	for (auto sub : _subscriptions)
		sub->push(db);
	return _subscriptions.size();
}

DataBuffer::DataBuffer()
{
}

DataBuffer::Pin::Pin()
	: data(nullptr),
	  memoryType(util::MemoryType::System)
{
}

DataBuffer::Pin::Pin(const void* d, util::MemoryType m)
	: data(d),
	  memoryType(m)
{
}

const DataBuffer::Pin& DataBuffer::getPin(util::Unique key) const
{
	auto pt = _pins.find(key);
	if (pt == _pins.end())
	{
		std::cerr << "Pin " << util::StringID::get(key) << " does not exist" <<
			std::endl;
		return _nullPin;
	}
	return _pins.find(key)->second;
}

void DataBuffer::release()
{
	releaseFunction();
}

