#include <slug/sim/Parameterized.h>

using namespace slug::sim;

std::string 
Parameterized::getParameter(const std::string& name, 
	                        const std::string& defaultValue)
{
	char* v = getParameterArray<char>(name);
	if (nullptr == v)
		return defaultValue;
	unsigned int length = *(unsigned int*)v;
	v += sizeof(unsigned int);
	return std::string(v, length);
}

bool Parameterized::setParameter(const std::string& name, void* values)
{
	_parameters[name] = values;
	return true;
}

bool Parameterized::require(const std::string& s)
{
	_required.push_back(s);
	return true;
}

bool Parameterized::validate()
{
	bool result = true;
	for (auto& r : _required)
	{
		if (_parameters.find(r) == _parameters.end())
		{
			std::cout << "Could not find parameter " << r << std::endl;
			result = false;
		}
	}
	return result;
}

bool Parameterized::read(util::InputStream& bs)
{
	unsigned int count = 0;
	bs >> count;
	for (unsigned int i = 0; i < count; ++i)
	{
		std::string name;
		unsigned int size = 0;
		bs >> name;
		bs >> size;
		char* data = new char[size];
		bs.read(data, size);
		setParameter(name, data);
	}
	return true;
}

bool Parameterized::read(const void* _p)
{
	char* p = (char*)_p;
	unsigned int count = *(unsigned int*)p;
	p += sizeof(unsigned int);
	for (unsigned int i = 0; i < count; ++i)
	{
		unsigned int length = *(unsigned int*)p;
		p += sizeof(unsigned int);
		std::string name(p, length);
		p += length;
		unsigned int size = *(unsigned int*)p;
		p += sizeof(unsigned int);
		char* data = new char[size];
		std::copy(p, p + size, data);
		p += size;
		setParameter(name, data);
	}
	return true;
}

