#include <slug/util/BinaryStream.h>
#include <slug/spec/ElementReport.h>
#include <slug/spec/ElementPlugin.h>
#include <algorithm>

using namespace slug::spec;
using namespace slug;

bool ElementReport::write(util::OutputStream& bs)
{
	//Machine->Reportable->Elements
	std::map<util::Unique, std::map<util::Unique, ElementList> > elementMap;

	//Map every element to sort them
	for (auto element : elements)
	{
		util::Unique machine = element->location.machine();
		util::Unique source = sourceID(element);
		elementMap[machine][source].push_back(element);
	}

	//Go through every machine
	for (auto it : elementMap)
	{
		//Write out the machine ID
		bs << it.first;	
		
		auto machineSet = it.second;

		//Then the number of reportables
		bs << (unsigned int)machineSet.size();

		//Go through every reportable needed on that machine
		for (auto jt : machineSet)
		{
			util::Unique source = jt.first;
			auto& elements = jt.second;

			//Write the reportable ID
			bs << source;

			//Write the number of elements
			bs << (unsigned int)elements.size();

			//Then every element's ID
			for (auto element : elements)
			{
				bs << elementID(element);
			}

			//TODO: Deprecated
			for (auto element : elements)
			{
				//bs << element->userID;
				bs << (unsigned int)0;
			}
		}
	}
	return true;
}

bool ElementReport::finalize()
{
	//Machine to element count
	std::map<util::Unique, unsigned int> count;
	for (auto element : elements)
		count[element->location.machine()]++;
	
	if (count.empty())
		return false;

	//Select the machine with the most elements so we have less to send
	auto winner = 
		std::max_element(count.begin(), count.end(),
		                 [](const std::pair<util::Unique, unsigned int>& a, 
						    const std::pair<util::Unique, unsigned int>& b)
						 {
						 	return a.second < b.second;
						 });

	//Make that machine the master
	master = winner->first;

	//Add all participating machine IDs to the report
	for (auto it : count)
		machines.push_back(it.first);
	
	return true;
}
unsigned int PluginElementReport::elementID(const Element* element) const
{
	return element->pluginID;
}

util::Unique PluginElementReport::sourceID(const Element* element) const
{
	return element->location.plugin->id;
}

unsigned int DeviceElementReport::elementID(const Element* element) const
{
	return element->deviceID();
}

util::Unique DeviceElementReport::sourceID(const Element* element) const
{
	return element->location.device();
}

unsigned int MachineElementReport::elementID(const Element* element) const
{
	return element->machineID();
}

util::Unique MachineElementReport::sourceID(const Element* element) const
{
	return element->location.machine();
}

unsigned int GlobalElementReport::elementID(const Element* element) const
{
	return element->globalID();
}

util::Unique GlobalElementReport::sourceID(const Element* element) const
{
	return master;
}

