#include <slug/spec/NeuronPlugin.h>

using namespace slug::spec;
using namespace slug;

NeuronPlugin::NeuronPlugin(util::Unique _type)
	: ElementPlugin(_type)
{
}

auto neuronUnique = util::StringID::get("neuron");

util::Unique NeuronPlugin::genre() const
{
	return neuronUnique;
}

bool NeuronPlugin::addNeuron(Neuron* n)
{
	neurons.push_back(n);
	n->location.plugin = this;
	return true;
}

bool NeuronPlugin::write(util::OutputStream& bs)
{
	std::vector<unsigned int> numSegments;
	std::vector<unsigned int> segmentStarts;
	std::vector<unsigned int> segmentEnds;
	for (auto neuron : neurons)
	{
		auto& synapses = neuron->incomingSynapses;
		std::map<util::Unique, SynapseList> synapseMap;
		for (auto synapse : synapses)
			synapseMap[synapse->type].push_back(synapse);
		numSegments.push_back(synapseMap.size());
		for (auto dt : synapseMap)
		{
			segmentStarts.push_back(dt.second.front()->deviceID());
			segmentEnds.push_back(dt.second.back()->deviceID());
		}
	}
	for (auto v : numSegments)
		bs << v;
	for (auto v : segmentStarts)
		bs << v;
	for (auto v : segmentEnds)
		bs << v;
	return true;
}

unsigned int NeuronPlugin::count() const
{
	return neurons.size();
}

Element* NeuronPlugin::element(unsigned int index)
{
	return neurons[index];
}
