#include <slug/spec/Schema.h>

using namespace slug::spec;
using namespace slug;

//Static variable instantiations
std::map<util::Unique, std::map<util::Unique, Schema*>> Schema::_schemas;
Schema Schema::_emptySchema;

Schema::Schema()
	: _numParameters(0)
{
}

unsigned int Schema::size() const
{
	return _numParameters;
}

bool Schema::add(Extractor* e)
{
	_extractors.push_back(e);
	_numParameters += e->numParameters();
	return true;
}

bool Schema::write(ElementList& el, util::OutputStream& os)
{
	bool result = true;
	for (auto e : _extractors)
	{
		result &= e->serialize(el, os);
	}
	return result;
}

bool Schema::add(Extractor* e, util::Unique genre, util::Unique type)
{
	//Create the schema if it doesn't already exist
	if (nullptr == _schemas[genre][type])
		_schemas[genre][type] = new Schema;

	_schemas[genre][type]->add(e);
}

Schema* Schema::get(util::Unique genre, util::Unique type)
{
	//Check that the genre exists
	if (_schemas.count(genre) == 0)
	{
		std::cout << "Warning: Schema for " << 
			util::StringID::get(genre) << ":" << 
			util::StringID::get(type) << " not found." <<
			" Using empty schema" << std::endl;
		return &_emptySchema;
	}

	//Check that the type within the genre exists
	if (_schemas[genre].count(type) == 0)
	{
		std::cout << "Warning: Schema for " << 
			util::StringID::get(genre) << ":" << 
			util::StringID::get(type) << " not found." <<
			" Using empty schema" << std::endl;
		return &_emptySchema;
	}

	return _schemas[genre][type];
}
