#include <slug/spec/Specification.h>

using namespace slug;
using namespace slug::spec;

Parameter<std::string>::Parameter(const std::string& value)
	: _value(value)
{
}

BaseParameter* Parameter<std::string>::copy() const
{
	return new Parameter(_value);
}

size_t Parameter<std::string>::size() const
{
	//Format is number of chars (uint) followed by the contents
	return sizeof(unsigned int) + _value.length();
}

bool Parameter<std::string>::write(std::ostream& os) const
{
	unsigned int s = _value.length();
	os.write((char*)&s, sizeof(s));
	os.write(_value.c_str(), _value.length());
	return os.good();
}

ArrayParameter::ArrayParameter(void* v, size_t s)
{
	_data = (char*)v;
	_size = s;
}

BaseParameter* ArrayParameter::copy() const
{
	char* data = new char[_size];
	std::copy((char*)_data, (char*)_data + _size, data);
	return new ArrayParameter(data, _size);
}

size_t ArrayParameter::size() const
{
	///Format is the contents
	return _size;
}

bool ArrayParameter::write(std::ostream& os) const
{
	os.write(_data, _size);
	return os.good();
}

ArrayParameter::~ArrayParameter()
{
	delete [] _data;
}

Specification::Specification()
{
}

bool Specification::set(util::Unique id, const std::string& v)
{
	if (_parameters.find(id) != _parameters.end())
	{
		delete _parameters[id];
	}
	_parameters[id] = new Parameter<std::string>(v);
	return true;
}

bool Specification::set(util::Unique id, const char* v)
{
	if (_parameters.find(id) != _parameters.end())
	{
		delete _parameters[id];
	}
	_parameters[id] = new Parameter<std::string>(v);
	return true;
}

bool Specification::set(const std::string& name, const std::string& v)
{
	util::Unique id = util::StringID::get(name);
	return set(id, v);
}

bool Specification::set(const std::string& name, const char* v)
{
	util::Unique id = util::StringID::get(name);
	return set(id, v);
}

const BaseParameter* Specification::get(util::Unique id) const
{
	if (_parameters.find(id) != _parameters.end())
		return _parameters.find(id)->second;
	return nullptr;
}

const BaseParameter* Specification::get(const std::string& name)
{
	util::Unique id = util::StringID::get(name);
	return get(id);
}

const size_t Specification::numParameters() const
{
	return _parameters.size();
}

std::vector<util::Unique> Specification::parameterIDs() const
{
	std::vector<util::Unique> result;
	for (auto it : _parameters)
	{
		result.push_back(it.first);
	}
	return result;
}

Specification::Specification(const Specification& s)
{
	//Make a copy of all parameters
	for (auto it : s._parameters)
	{
		_parameters[it.first] = it.second->copy();
	}
}

