#include <slug/util/Memory.h>
#include <cuda_runtime.h>
#include <string.h>
#include <slug/sim/Device.h>
#include <slug/sim/CUDA.h>
#include <slug/util/Debug.h>
#include <stdexcept>

using namespace slug::util;

template<>
void* allocate<MemoryType::CUDA>(size_t size)
{
	char* result;
	cudaSafeCall(cudaMalloc((void**)&result, size));
	if (nullptr == result)
	{
		std::cerr << "Out of CUDA Memory" << std::endl;
		throw std::runtime_error("Out of CUDA Memory");
	}
	//cudaPrintPointer(result);
	return result;
}

template<>
void* allocateHost<MemoryType::CUDA>(size_t size)
{
	char* result;
	cudaSafeCall(cudaMallocHost((void**)&result, size));
	return result;
}

template<>
void deallocate<MemoryType::CUDA>(void* ptr)
{
	cudaSafeCall(cudaFree(ptr));
}

template<>
void zero<MemoryType::CUDA>(void* ptr, size_t size)
{
	cudaSafeCall(cudaMemsetAsync(ptr, 0, size, sim::CUDA::stream));
	//cudaSafeCall(cudaStreamSynchronize(sim::CUDA::stream));
}

template<>
void* allocate<MemoryType::System>(size_t size)
{
	return new char[size];
}

template<>
void* allocateHost<MemoryType::System>(size_t size)
{
	char* result;
	cudaSafeCall(cudaMallocHost((void**)&result, size));
	return result;
}

template<>
void deallocate<MemoryType::System>(void* ptr)
{
	delete [] (char*)ptr;
}

template<>
void zero<MemoryType::System>(void* ptr, size_t size)
{
	memset(ptr, 0, size);
}

template<>
bool transfer<MemoryType::System, MemoryType::System>
	(void* dst, void* src, size_t count)
{
	memcpy(dst, src, count);
	return true;
}

template<>
bool transfer<MemoryType::CUDA, MemoryType::CUDA>
	(void* dst, void* src, size_t count)
{
	cudaSafeCall(cudaMemcpyAsync(dst, src, count, 
	                             cudaMemcpyDeviceToDevice, sim::CUDA::stream));
	//cudaSafeCall(cudaStreamSynchronize(sim::CUDA::stream));
	//cudaSafeCall(cudaMemcpy(dst, src, count, cudaMemcpyDeviceToDevice));
	return true;
}

template<>
bool transfer<MemoryType::CUDA, MemoryType::System>
	(void* dst, void* src, size_t count)
{
	cudaSafeCall(cudaMemcpyAsync(dst, src, count, 
		                         cudaMemcpyHostToDevice, sim::CUDA::stream));
	//cudaSafeCall(cudaStreamSynchronize(sim::CUDA::stream));
	//cudaSafeCall(cudaMemcpy(dst, src, count, cudaMemcpyHostToDevice));
	return true;
}

template<>
bool transfer<MemoryType::System, MemoryType::CUDA>
	(void* dst, void* src, size_t count)
{
	cudaSafeCall(cudaMemcpyAsync(dst, src, count, 
	                cudaMemcpyDeviceToHost, sim::CUDA::stream));
	//cudaSafeCall(cudaStreamSynchronize(sim::CUDA::stream));
	//cudaSafeCall(cudaMemcpy(dst, src, count, cudaMemcpyDefault));
	return true;
}
